# -*- coding: utf-8 -*-
import pytest
import sys
import time
import cPickle as pickle
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestRemovebin(object):
    def setup_class(cls):
        """
        Setup class.
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            TestRemovebin.client = aerospike.client(config).connect()
        else:
            TestRemovebin.client = aerospike.client(config).connect(user,
                                                                    password)

    def teardown_class(cls):
        TestRemovebin.client.close()

    def setup_method(self, method):
        """
        Setup method.
        """
        for i in xrange(5):
            key = ('test', 'demo', i)
            rec = {'name': 'name%s' % (str(i)), 'age': i}
            TestRemovebin.client.put(key, rec)

    def teardown_method(self, method):
        """
        Teardoen method.
        """
        for i in xrange(5):
            key = ('test', 'demo', i)
            try:
                (key , meta, bins) = TestRemovebin.client.get(key)
            except RecordNotFound:
                TestRemovebin.client.remove(key)

    def test_remove_bin_with_no_parameters(self):
        """
        Invoke remove_bin() without any mandatory parameters.
        """
        with pytest.raises(TypeError) as typeError:
            TestRemovebin.client.remove_bin()
        assert "Required argument 'key' (pos 1) not found" in typeError.value

    def test_remove_bin_with_correct_parameters(self):
        """
        Invoke remove_bin() with correct parameters
        """
        key = ('test', 'demo', 1)
        TestRemovebin.client.remove_bin(key, ["age"])

        (key, meta, bins) = TestRemovebin.client.get(key)

        assert bins == {'name': 'name1'}

    def test_remove_bin_with_correct_policy(self):
        """
        Invoke remove_bin() with correct policy
        """
        key = ('test', 'demo', 1)
        policy = {'timeout': 1000}
        TestRemovebin.client.remove_bin(key, ["age"], {}, policy)

        (key, meta, bins) = TestRemovebin.client.get(key)

        assert bins == {'name': 'name1'}

    def test_remove_bin_with_policy_send_gen_ignore(self):
        """
        Invoke remove_bin() with policy send
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'key': aerospike.POLICY_KEY_SEND,
            'gen': aerospike.POLICY_GEN_IGNORE
        }
        meta = {'gen': 2, 'ttl': 1000}
        TestRemovebin.client.remove_bin(key, ["age"], meta, policy)

        (key, meta, bins) = TestRemovebin.client.get(key)

        assert bins == {'name': 'name1'}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_remove_bin_with_policy_send_gen_eq_positive(self):
        """
        Invoke remove_bin() with policy gen eq less
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'key': aerospike.POLICY_KEY_SEND,
            'gen': aerospike.POLICY_GEN_EQ
        }

        (key, meta) = TestRemovebin.client.exists(key)
        gen = meta['gen']
        meta = {'gen': gen, 'ttl': 1000}

        TestRemovebin.client.remove_bin(key, ["age"], meta, policy)

        (key, meta, bins) = TestRemovebin.client.get(key)

        assert bins == {'name': 'name1'}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_remove_bin_with_policy_send_gen_eq_not_equal(self):
        """
        Invoke remove_bin() with policy gen eq not equal
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'key': aerospike.POLICY_KEY_SEND,
            'gen': aerospike.POLICY_GEN_EQ
        }
        (key, meta) = TestRemovebin.client.exists(key)
        gen = meta['gen']
        meta = {'gen': gen + 5, 'ttl': 1000}

        try:
            TestRemovebin.client.remove_bin(key, ["age"], meta, policy)

        except RecordGenerationError as exception:
            assert exception.code == 3
            assert exception.msg == "AEROSPIKE_ERR_RECORD_GENERATION"

        (key, meta, bins) = TestRemovebin.client.get(key)

        assert bins == {'age': 1, 'name': 'name1'}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_remove_bin_with_policy_send_gen_GT_lesser(self):
        """
        Invoke remove_bin() with policy gen GT lesser
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'key': aerospike.POLICY_KEY_SEND,
            'gen': aerospike.POLICY_GEN_GT
        }

        (key, meta) = TestRemovebin.client.exists(key)
        gen = meta['gen']
        meta = {'gen': gen, 'ttl': 1000}

        try:
            TestRemovebin.client.remove_bin(key, ["age"], meta, policy)

        except RecordGenerationError as exception:
            assert exception.code == 3
            assert exception.msg == "AEROSPIKE_ERR_RECORD_GENERATION"

        (key, meta, bins) = TestRemovebin.client.get(key)

        assert bins == {'age': 1, 'name': 'name1'}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_remove_bin_with_policy_send_gen_GT_positive(self):
        """
        Invoke remove_bin() with policy gen GT positive
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'key': aerospike.POLICY_KEY_SEND,
            'gen': aerospike.POLICY_GEN_GT
        }

        (key, meta) = TestRemovebin.client.exists(key)
        gen = meta['gen']
        meta = {'gen': gen + 5, 'ttl': 1000}

        TestRemovebin.client.remove_bin(key, ["age"], meta, policy)

        (key, meta, bins) = TestRemovebin.client.get(key)

        assert bins == {'name': 'name1'}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_remove_bin_with_policy_key_digest(self):
        """
        Invoke remove_bin() with policy key digest
        """
        key = ('test', 'demo', None, bytearray("asd;as[d'as;djk;uyfl",
                                               "utf-8"))
        rec = {'age': 1, 'name': 'name1'}
        TestRemovebin.client.put(key, rec)
        policy = {'timeout': 1000, 'key': aerospike.POLICY_KEY_DIGEST}
        TestRemovebin.client.remove_bin(key, ["age"], {}, policy)

        (key, meta, bins) = TestRemovebin.client.get(key)

        assert bins == {'name': 'name1'}
        assert key == ('test', 'demo', None,
                       bytearray(b"asd;as[d\'as;djk;uyfl"))

        TestRemovebin.client.remove(key)

    def test_remove_bin_with_incorrect_policy(self):
        """
        Invoke remove_bin() with incorrect policy
        """
        key = ('test', 'demo', 1)

        policy = {
            'timeout': 0.5
        }
        try:
            TestRemovebin.client.remove_bin(key, ["age"], {}, policy)

        except ClientError as exception:
            assert exception.code == -1
            assert exception.msg == "Incorrect policy"

    def test_remove_bin_with_nonexistent_key(self):
        """
        Invoke remove_bin() with non-existent key
        """
        key = ('test', 'demo', "non-existent")
        status = TestRemovebin.client.remove_bin(key, ["age"])

        assert status == 0L

    def test_remove_bin_with_nonexistent_bin(self):
        """
        Invoke remove_bin() with non-existent bin
        """
        key = ('test', 'demo', 1)
        status = TestRemovebin.client.remove_bin(key, ["non-existent"])

        assert status == 0L

    def test_remove_bin_with_single_bin_in_a_record(self):
        """
        Invoke remove_bin() with policy key digest
        """
        key = ('test', 'demo', "single-bin")
        try:
            TestRemovebin.client.remove(key)
        except:
            pass

        rec = {'name': 'single'}
        TestRemovebin.client.put(key, rec)

        policy = {'timeout': 1000}
        TestRemovebin.client.remove_bin(key, ["name"], {}, policy)

        _, _, bins = TestRemovebin.client.get(key)
        assert bins == None


    def test_remove_bin_with_extra_parameter(self):
        """
        Invoke remove_bin() with extra parameter.
        """
        key = ('test', 'demo', 1)
        policy = {'timeout': 1000}
        with pytest.raises(TypeError) as typeError:
            TestRemovebin.client.remove_bin(key, ["age"], {}, policy, "")

        assert "remove_bin() takes at most 4 arguments (5 given)" in typeError.value

    def test_remove_bin_key_is_none(self):
        """
        Invoke remove_bin() with key is none
        """
        try:
            TestRemovebin.client.remove_bin(None, ["age"])

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "key is invalid"

    def test_remove_bin_bin_is_none(self):
        """
        Invoke remove_bin() with bin is none
        """
        key = ('test', 'demo', 1)
        try:
            TestRemovebin.client.remove_bin(key, None)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Bins should be a list"

    def test_remove_bin_no_bin(self):
        """
        Invoke remove_bin() no bin
        """
        key = ('test', 'demo', 1)
        TestRemovebin.client.remove_bin(key, [])

        (key , meta, bins) = TestRemovebin.client.get(key)

        assert bins == { 'name': 'name1', 'age': 1}

    def test_remove_bin_all_bins(self):
        """
        Invoke remove_bin() all bins
        """
        key = ('test', 'demo', 1)
        TestRemovebin.client.remove_bin(key, ["name", "age"])


        try:
            (key , meta, bins) = TestRemovebin.client.get(key)

        except RecordNotFound as exception:
            assert exception.code == 2
        for i in xrange(5):
            key = ('test', 'demo', i)
            rec = {
                'name' : 'name%s' % (str(i)),
                'age' : i
            }
            TestRemovebin.client.put(key, rec)

    def test_remove_bin_with_unicode_binname(self):
        """
        Invoke remove_bin() with unicode bin name
        """
        key = ('test', 'demo', 2)

        TestRemovebin.client.remove_bin(key, [u"name"])

        (key, meta, bins) = TestRemovebin.client.get(key)

        assert bins == {'age': 2}

        key = ('test', 'demo', 3)

        TestRemovebin.client.remove_bin(key, [u"name", "age"])

        try:
            (key , meta, bins) = TestRemovebin.client.get(key)

        except RecordNotFound as exception:
            assert exception.code == 2
        key = ('test', 'demo', 4)

        TestRemovebin.client.remove_bin(key, ["name", u"age"])

        try:
            (key , meta, bins) = TestRemovebin.client.get(key)

        except RecordNotFound as exception:
            assert exception.code == 2
        for i in xrange(5):
            key = ('test', 'demo', i)
            rec = {
                'name' : 'name%s' % (str(i)),
                'age' : i
            }
            TestRemovebin.client.put(key, rec)

    def test_remove_bin_with_correct_parameters_without_connection(self):
        """
        Invoke remove_bin() with correct parameters without connection
        """
        config = {'hosts': [('127.0.0.1', 3000)]}
        client1 = aerospike.client(config)

        key = ('test', 'demo', 1)

        try:
            client1.remove_bin(key, ["age"])

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'



# -*- coding: utf-8 -*-

import pytest
import sys
import cPickle as pickle
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestIndex(TestBaseClass):
    def setup_class(cls):
        """
        Setup method.
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            TestIndex.client = aerospike.client(config).connect()
        else:
            TestIndex.client = aerospike.client(config).connect(user, password)

    def teardown_class(cls):
        TestIndex.client.close()

    def setup_method(self, method):
        """
        Setup method. 
        """

        for i in xrange(5):
            key = ('test', u'demo', i)
            rec = {
                'name': 'name%s' % (str(i)),
                'addr': 'name%s' % (str(i)),
                'age': i,
                'no': i
            }
            TestIndex.client.put(key, rec)

    def teardown_method(self, method):
        """
        Teardoen method.
        """
        for i in xrange(5):
            key = ('test', u'demo', i)
            TestIndex.client.remove(key)

    def test_creatindex_with_no_paramters(self):
        """
            Invoke indexcreate() without any mandatory parameters.
        """
        with pytest.raises(TypeError) as typeError:
            TestIndex.client.index_string_create()

        assert "Required argument 'ns' (pos 1) not found" in typeError.value

    def test_createindex_with_correct_parameters(self):
        """
            Invoke createindex() with correct arguments
        """
        policy = {}
        retobj = TestIndex.client.index_integer_create('test', 'demo', 'age',
                                                       'age_index', policy)

        assert retobj == 0L
        TestIndex.client.index_remove('test', 'age_index', policy)

    def test_createindex_with_correct_parameters_set_length_extra(self):
            #Invoke createindex() with correct arguments
        set_name = 'a'
        for i in xrange(100):
            set_name = set_name + 'a'
        policy = {}
        with pytest.raises(Exception) as exception:
            retobj = TestIndex.client.index_integer_create('test', set_name,
'age', 'age_index', policy)

        assert exception.value[0] == 4

    def test_createindex_with_incorrect_namespace(self):
        """
            Invoke createindex() with incorrect namespace
        """
        policy = {}
        try:
            retobj = TestIndex.client.index_integer_create( 'test1', 'demo',
'age', 'age_index', policy )

        except InvalidRequest as exception:
            assert exception.code == 4

    def test_createindex_with_incorrect_set(self):
        """
            Invoke createindex() with incorrect set
        """
        policy = {}
        retobj = TestIndex.client.index_integer_create('test', 'demo1', 'age',
                                                       'age_index', policy)

        assert retobj == 0L
        TestIndex.client.index_remove('test', 'age_index', policy)

    def test_createindex_with_incorrect_bin(self):
        """
            Invoke createindex() with incorrect bin
        """
        policy = {}
        retobj = TestIndex.client.index_integer_create('test', 'demo', 'age1',
                                                       'age_index', policy)

        assert retobj == 0L
        TestIndex.client.index_remove('test', 'age_index', policy)

    def test_createindex_with_namespace_is_none(self):
        """
            Invoke createindex() with namespace is None
        """
        policy = {}
        try:
            retobj = TestIndex.client.index_integer_create( None, 'demo',
'age', 'age_index', policy )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'Namespace should be a string'

    def test_createindex_with_set_is_none(self):
            #Invoke createindex() with set is None
        policy = {}

        retobj = TestIndex.client.index_integer_create( 'test', None,
'age', 'age_index' , policy)

        assert retobj == 0L
        TestIndex.client.index_remove('test', 'age_index', policy);

    def test_createindex_with_set_is_int(self):
            #Invoke createindex() with set is int
        policy = {}
        with pytest.raises(Exception) as exception:
            retobj = TestIndex.client.index_integer_create('test', 1, 'age',
                                                           'age_index', policy)
        assert exception.value[0] == -2
        assert exception.value[1] == 'Set should be string, unicode or None'

    def test_createindex_with_bin_is_none(self):
        """
            Invoke createindex() with bin is None
        """
        policy = {}
        try:
            retobj = TestIndex.client.index_integer_create( 'test', 'demo',
None, 'age_index' , policy)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'Bin should be a string'

    def test_createindex_with_index_is_none(self):
        """
            Invoke createindex() with index is None
        """
        policy = {}
        try:
            retobj = TestIndex.client.index_integer_create( 'test', 'demo',
'age', None, policy )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'Index name should be string or unicode'

    def test_create_same_index_multiple_times(self):
        """
            Invoke createindex() with multiple times on same bin
        """
        policy = {}
        retobj = TestIndex.client.index_integer_create('test', 'demo', 'age',
                                                       'age_index', policy)
        if retobj == 0L:
            retobj = TestIndex.client.index_integer_create('test', 'demo',
                                                           'age', 'age_index',
                                                           policy)
            TestIndex.client.index_remove('test', 'age_index', policy)
            assert retobj == 0L
        else:
            assert True == False

    def test_create_same_index_multiple_times_different_bin(self):
        """
            Invoke createindex() with multiple times on different bin
        """
        policy = {}
        retobj = TestIndex.client.index_integer_create('test', 'demo', 'age',
                                                       'age_index', policy)
        if retobj == 0L:
            retobj = TestIndex.client.index_integer_create('test', 'demo',
                                                           'no', 'age_index',
                                                           policy)
            assert retobj == 0L
            TestIndex.client.index_remove('test', 'age_index', policy)
        else:
            assert True == False

    def test_create_different_index_multiple_times_same_bin(self):
        """
            Invoke createindex() with multiple times on same bin with different
name
        """
        policy = {}
        retobj = TestIndex.client.index_integer_create('test', 'demo', 'age',
                                                       'age_index', policy)
        if retobj == 0L:
            retobj = TestIndex.client.index_integer_create('test', 'demo',
                                                           'age', 'age_index1',
                                                           policy)
            assert retobj == 0L
            TestIndex.client.index_remove('test', 'age_index', policy)
            TestIndex.client.index_remove('test', 'age_index1', policy)
        else:
            assert True == False

    def test_createindex_with_policy(self):
        """
            Invoke createindex() with policy
        """
        policy = {'timeout': 1000}
        retobj = TestIndex.client.index_integer_create('test', 'demo', 'age',
                                                       'age_index', policy)

        assert retobj == 0L
        TestIndex.client.index_remove('test', 'age_index', policy)

    def test_create_string_index_positive(self):
        """
            Invoke create string index() with correct arguments
        """
        policy = {}
        retobj = TestIndex.client.index_string_create('test', 'demo', 'name',
                                                      'name_index', policy)

        assert retobj == 0L
        TestIndex.client.index_remove('test', 'name_index', policy)

    def test_create_string_index_with_correct_parameters_set_length_extra(self):
            #Invoke createindex() with correct arguments set length extra
        set_name = 'a'
        for i in xrange(100):
            set_name = set_name + 'a'
        policy = {}
        with pytest.raises(Exception) as exception:
            retobj = TestIndex.client.index_string_create('test', set_name,
'name', 'name_index', policy)

        assert exception.value[0] == 4

    def test_create_string_index_with_correct_parameters_ns_length_extra(self):
            #Invoke createindex() with correct arguments ns length extra
        ns_name = 'a'
        for i in xrange(50):
            ns_name = ns_name + 'a'
        policy = {}
        with pytest.raises(Exception) as exception:
            retobj = TestIndex.client.index_string_create(ns_name, 'demo',
'name', 'name_index', policy)

        assert exception.value[0] == 4

    def test_create_string_index_with_incorrect_namespace(self):
        """
            Invoke create string index() with incorrect namespace
        """
        policy = {}
        try:
            retobj = TestIndex.client.index_string_create('test1', 'demo',
'name', 'name_index', policy)

        except InvalidRequest as exception:
            assert exception.code == 4

    def test_create_string_index_with_incorrect_set(self):
        """
            Invoke create string index() with incorrect set
        """
        policy = {}
        retobj = TestIndex.client.index_string_create('test', 'demo1', 'name',
                                                      'name_index', policy)

        assert retobj == 0L
        TestIndex.client.index_remove('test', 'name_index', policy)

    def test_create_string_index_with_incorrect_bin(self):
        """
            Invoke create string index() with incorrect bin
        """
        policy = {}
        retobj = TestIndex.client.index_string_create('test', 'demo', 'name1',
                                                      'name_index', policy)

        assert retobj == 0L
        TestIndex.client.index_remove('test', 'name_index', policy)

    def test_create_string_index_with_namespace_is_none(self):
        """
            Invoke create string index() with namespace is None
        """
        policy = {}
        try:
            retobj = TestIndex.client.index_string_create( None, 'demo', 'name', 'name_index', policy )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'Namespace should be a string'

    def test_create_string_index_with_set_is_none(self):
            #Invoke create string index() with set is None
        policy = {}
        retobj = TestIndex.client.index_string_create('test', None, 'name',
                                                          'name_index', policy)

        assert retobj == 0L
        TestIndex.client.index_remove('test', 'name_index', policy);

    def test_create_string_index_with_bin_is_none(self):
        """
            Invoke create string index() with bin is None
        """
        policy = {}
        try:
            retobj = TestIndex.client.index_string_create('test', 'demo',
None, 'name_index', policy)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'Bin should be a string'

    def test_create_string_index_with_index_is_none(self):
        """
            Invoke create_string_index() with index is None
        """
        policy = {}
        try:
            retobj = TestIndex.client.index_string_create('test', 'demo',
'name', None, policy)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'Index name should be string or unicode'

    def test_create_same_string_index_multiple_times(self):
        """
            Invoke create string index() with multiple times on same bin
        """
        policy = {}
        retobj = TestIndex.client.index_string_create('test', 'demo', 'name',
                                                      'name_index', policy)
        if retobj == 0L:
            retobj = TestIndex.client.index_string_create('test', 'demo',
                                                          'name', 'name_index',
                                                          policy)
            assert retobj == 0L
            TestIndex.client.index_remove('test', 'name_index', policy)
        else:
            assert True == False

    def test_create_same_string__index_multiple_times_different_bin(self):
        """
            Invoke create string index() with multiple times on different bin
        """
        policy = {}
        retobj = TestIndex.client.index_string_create('test', 'demo', 'name',
                                                      'name_index', policy)
        if retobj == 0L:
            retobj = TestIndex.client.index_string_create('test', 'demo',
                                                          'addr', 'name_index',
                                                          policy)
            assert retobj == 0L
            TestIndex.client.index_remove('test', 'name_index', policy)
        else:
            assert True == False

    def test_create_different_string_index_multiple_times_same_bin(self):
        """
            Invoke create string index() with multiple times on same bin with different
name
        """
        policy = {}
        retobj = TestIndex.client.index_string_create('test', 'demo', 'name',
                                                      'name_index', policy)
        if retobj == 0L:
            retobj = TestIndex.client.index_string_create(
                'test', 'demo', 'name', 'name_index1', policy)
            assert retobj == 0L
            TestIndex.client.index_remove('test', 'name_index', policy)
            TestIndex.client.index_remove('test', 'name_index1', policy)
        else:
            assert True == False

    def test_create_string_index_with_policy(self):
        """
            Invoke create string index() with policy
        """
        policy = {'timeout': 1000}
        retobj = TestIndex.client.index_string_create('test', 'demo', 'name',
                                                      'name_index', policy)

        assert retobj == 0L
        TestIndex.client.index_remove('test', 'name_index', policy)

    def test_drop_invalid_index(self):
        """
            Invoke drop invalid index() 
        """
        policy = {}
        retobj = TestIndex.client.index_remove('test', 'age_index_new', policy)
        assert retobj == 0L

    def test_drop_valid_index(self):
        """
            Invoke drop valid index()
        """
        policy = {}
        retobj = TestIndex.client.index_integer_create('test', 'demo', 'age',
                                                       'age_index', policy)
        retobj = TestIndex.client.index_remove('test', 'age_index', policy)
        assert retobj == 0L

    def test_drop_valid_index_policy(self):
        """
            Invoke drop valid index() policy
        """
        policy = {'timeout': 1000}
        retobj = TestIndex.client.index_integer_create('test', 'demo', 'age',
                                                       'age_index', policy)
        retobj = TestIndex.client.index_remove('test', 'age_index', policy)
        assert retobj == 0L

    """
    This test case causes a db crash and hence has been commented. Work pending
on the C-client side
    def test_createindex_with_long_index_name(self):
            Invoke createindex() with long index name
        policy = {}
        retobj = TestIndex.client.index_integer_create( 'test', 'demo',
'age',
'bin2_integer_indexsdadadfasdfasdfeartfqrgahfasdfheudsdfasdfawf312342q3453rf9qwfasdcfasdcalskdcbacfq34915rwcfasdcascnabscbaskjdbcalsjkbcdasc', policy)

        assert retobj == 0L
        TestIndex.client.index_remove(policy, 'test',
'bin2_integer_indexsdadadfasdfasdfeartfqrgahfasdfheudsdfasdfawf312342q3453rf9qwfasdcfasdcalskdcbacfq34915rwcfasdcascnabscbaskjdbcalsjkbcdasc');
    """

    def test_create_string_index_unicode_positive(self):
        """
            Invoke create string index() with correct arguments
        """
        policy = {}
        retobj = TestIndex.client.index_string_create('test', u'demo', u'name',
                                                      u'uni_name_index',
                                                      policy)

        assert retobj == 0L
        TestIndex.client.index_remove('test', u'uni_name_index', policy)

    def test_createindex_integer_unicode(self):
        """
            Invoke createindex() with correct arguments
        """
        policy = {}
        retobj = TestIndex.client.index_integer_create('test', u'demo', u'age',
                                                       u'uni_age_index',
                                                       policy)

        assert retobj == 0L
        TestIndex.client.index_remove('test', u'age_index', policy)

    def test_createindex_with_correct_parameters_without_connection(self):
            #Invoke createindex() with correct arguments without connection
        policy = {}
        config = {'hosts': [('127.0.0.1', 3000)]}
        client1 = aerospike.client(config)

        try:
            etobj = client1.index_integer_create('test', 'demo', 'age', 'age_index', policy)

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'



# -*- coding: utf-8 -*-
import pytest
import time
import sys
import cPickle as pickle
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestOperate(object):
    def setup_class(cls):
        """
        Setup method.
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            TestOperate.client = aerospike.client(config).connect()
        else:
            TestOperate.client = aerospike.client(config).connect(user,
                                                                  password)
        config_no_typechecks = {'hosts': hostlist, 'strict_types': False}
        if user == None and password == None:
            TestOperate.client_no_typechecks = aerospike.client(config_no_typechecks).connect()
        else:
            TestOperate.client_no_typechecks = aerospike.client(config_no_typechecks).connect(user, password)

        TestOperate.skip_old_server = True
        versioninfo = TestOperate.client.info('version')
        for keys in versioninfo:
            for value in versioninfo[keys]:
                if value != None:
                    versionlist = value[value.find("build") + 6:value.find("\n")].split(".")
                    if int(versionlist[0]) >= 3 and int(versionlist[1]) >= 6:
                        TestOperate.skip_old_server = False

    def teardown_class(cls):
        TestOperate.client.close()
        TestOperate.client_no_typechecks.close()

    def setup_method(self, method):
        TestOperate.keys = []

        for i in xrange(5):
            key = ('test', 'demo', i)
            rec = {'name': 'name%s' % (str(i)), 'age': i}
            TestOperate.client.put(key, rec)
        key = ('test', 'demo', 6)
        rec = {"age": 6.3}
        TestOperate.client.put(key, rec)

        key = ('test', 'demo', 'bytearray_key')
        rec = {"bytearray_bin": bytearray("asd;as[d'as;d", "utf-8")}
        TestOperate.client.put(key, rec)

        TestOperate.keys.append(key)

        key = ('test', 'demo', 'existing_key')
        rec = {"dict": {"a": 1}, "bytearray": bytearray("abc"), "float": 3.4,
                "list": ['a']}
        TestOperate.client.put(key, rec)

        TestOperate.keys.append(key)

    def teardown_method(self, method):
        """
        Teardoen method.
        """
        for i in xrange(5):
            key = ('test', 'demo', i)
            try:
                TestOperate.client.remove(key)
            except RecordNotFound as exception:
                pass
        for key in TestOperate.keys:
            try:
                TestOperate.client.remove(key)
            except RecordNotFound as exception:
                pass

    def test_operate_with_no_parameters_negative(self):
        """
        Invoke opearte() without any mandatory parameters.
        """
        with pytest.raises(TypeError) as typeError:
            TestOperate.client.operate()
        assert "Required argument 'key' (pos 1) not found" in typeError.value

    def test_operate_with_correct_paramters_positive(self):
        """
        Invoke operate() with correct parameters
        """
        key = ('test', 'demo', 1)
        list = [
            {"op": aerospike.OPERATOR_PREPEND,
             "bin": "name",
             "val": u"ram"},
            {"op": aerospike.OPERATOR_INCR,
             "bin": "age",
             "val": 3}, {"op": aerospike.OPERATOR_READ,
                         "bin": "name"}
        ]

        key, meta, bins = TestOperate.client.operate(key, list)

        assert bins == {'name': 'ramname1'}

    def test_operate_with_increment_positive_float_value(self):
        """
        Invoke operate() with correct parameters
        """
        if TestOperate.skip_old_server == True:
            pytest.skip("Server does not support increment on float type")
        key = ('test', 'demo', 6)
        list = [
            {"op": aerospike.OPERATOR_INCR,
             "bin": "age",
             "val": 3.5}, 
            {"op": aerospike.OPERATOR_READ,
             "bin": "age"}
        ]

        key, meta, bins = TestOperate.client.operate(key, list)

        assert bins == {'age': 9.8}

    def test_operate_with_correct_policy_positive(self):
        """
        Invoke operate() with correct policy
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'commit_level': aerospike.POLICY_COMMIT_LEVEL_MASTER
        }

        list = [{"op": aerospike.OPERATOR_APPEND,
                 "bin": "name",
                 "val": "aa"},
                {"op": aerospike.OPERATOR_INCR,
                 "bin": "age",
                 "val": 3}, {"op": aerospike.OPERATOR_READ,
                             "bin": "name"}]

        key, meta, bins = TestOperate.client.operate(key, list, {}, policy)

        assert bins == {'name': 'name1aa'}
        assert key == ('test', 'demo', 1, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

        TestOperate.client.remove(key)

    def test_operate_with_policy_key_digest(self):
        """
        Invoke operate() with correct policy
        """
        key = ('test', 'demo', None, bytearray("asd;as[d'as;djk;uyfl",
                                               "utf-8"))
        rec = {'name': 'name%s' % (str(1)), 'age': 1, }
        policy = {'timeout': 1000, 'key': aerospike.POLICY_KEY_DIGEST}
        TestOperate.client.put(key, rec)

        list = [{"op": aerospike.OPERATOR_APPEND,
                 "bin": "name",
                 "val": "aa"},
                {"op": aerospike.OPERATOR_INCR,
                 "bin": "age",
                 "val": 3}, {"op": aerospike.OPERATOR_READ,
                             "bin": "name"}]

        key, meta, bins = TestOperate.client.operate(key, list, {}, policy)

        assert bins == {'name': 'name1aa'}
        assert key == ('test', 'demo', None,
                       bytearray(b"asd;as[d\'as;djk;uyfl"))

    def test_operate_with_policy_gen_ignore(self):
        """
        Invoke operate() with gen ignore.
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'gen': aerospike.POLICY_GEN_IGNORE,
            'commit_level': aerospike.POLICY_COMMIT_LEVEL_ALL
        }

        meta = {'gen': 10, 'ttl': 1200}

        list = [{"op": aerospike.OPERATOR_APPEND,
                 "bin": "name",
                 "val": "aa"},
                {"op": aerospike.OPERATOR_INCR,
                 "bin": "age",
                 "val": 3}, {"op": aerospike.OPERATOR_READ,
                             "bin": "name"}]

        key, meta, bins = TestOperate.client.operate(key, list, meta, policy)

        assert bins == {'name': 'name1aa'}
        assert key == ('test', 'demo', 1, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_operate_with_policy_gen_EQ_positive(self):
        """
        Invoke operate() with gen EQ positive.
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'gen': aerospike.POLICY_GEN_EQ
        }
        (key, meta) = TestOperate.client.exists(key)
        gen = meta['gen']
        meta = {'gen': gen, 'ttl': 1200}

        list = [{"op": aerospike.OPERATOR_APPEND,
                 "bin": "name",
                 "val": "aa"},
                {"op": aerospike.OPERATOR_INCR,
                 "bin": "age",
                 "val": 3}, {"op": aerospike.OPERATOR_READ,
                             "bin": "name"}]

        (key, meta, bins) = TestOperate.client.operate(key, list, meta, policy)

        assert bins == {'name': 'name1aa'}
        assert key == ('test', 'demo', 1, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_operate_touch_operation_nobin_withvalue(self):
        """
        Invoke operate() with touch value. No bin specified. Value is specified
        """
        key = ('test', 'demo', 1)
        list = [
            {"op": aerospike.OPERATOR_TOUCH,
             "val": 4000}
        ]

        status = TestOperate.client.operate(key, list)

        (key, meta) = TestOperate.client.exists(key)

        assert meta['ttl'] != None

    def test_operate_touch_operation_withbin_withvalue(self):
        """
        Invoke operate() with touch operation. Bin and value both specified
        """
        key = ('test', 'demo', 1)
        list = [
            {"op": aerospike.OPERATOR_TOUCH,
             "bin": "age",
             "val": 4000}
        ]

        TestOperate.client.operate(key, list)

        (key, meta) = TestOperate.client.exists(key)

        assert meta['ttl'] != None

    def test_operate_touch_operation_withbin_novalue(self):
        """
        Invoke operate() with touch operation. Bin is specified but no value
        specified
        """
        key = ('test', 'demo', 1)
        list = [
            {"op": aerospike.OPERATOR_TOUCH,
             "bin": "age"}
        ]

        status = TestOperate.client.operate(key, list)

        (key, meta) = TestOperate.client.exists(key)

        assert meta['ttl'] != None

    def test_operate_touch_operation_nobin_novalue(self):
        """
        Invoke operate() with touch operation. Bin and value not specified
        """
        key = ('test', 'demo', 1)
        list = [
            {"op": aerospike.OPERATOR_TOUCH}
        ]

        status = TestOperate.client.operate(key, list)

        (key, meta) = TestOperate.client.exists(key)

        assert meta['ttl'] != None

    def test_operate_with_policy_gen_EQ_not_equal(self):
        """
        Invoke operate() with gen not equal.
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'gen': aerospike.POLICY_GEN_EQ
        }

        (key, meta) = TestOperate.client.exists(key)
        gen = meta['gen']
        meta = {
            'gen': gen + 5,
            'ttl': 1200
        }
        list = [
                {
                    "op" : aerospike.OPERATOR_APPEND,
                    "bin" : "name",
                    "val" : "aa"
                    },
                {
                    "op" : aerospike.OPERATOR_INCR,
                    "bin" : "age",
                    "val" : 3
                    },
                {
                    "op" : aerospike.OPERATOR_READ,
                    "bin" : "name"
                    }
                ]
        try:
            key, meta, bins = TestOperate.client.operate(key, list, meta, policy)

        except RecordGenerationError as exception:
            assert exception.code == 3L
       
        (key , meta, bins) = TestOperate.client.get(key)
        assert bins == { "age": 1, 'name': 'name1'}
        assert key == ('test', 'demo', None,
                bytearray(b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8'))
        
    def test_operate_with_policy_gen_GT_lesser(self):
        """
        Invoke operate() with gen GT lesser.
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'gen': aerospike.POLICY_GEN_GT
        }
        (key, meta) = TestOperate.client.exists(key)
        gen = meta['gen']
        meta = {'gen': gen, 'ttl': 1200}

        list = [{"op": aerospike.OPERATOR_APPEND,
                 "bin": "name",
                 "val": "aa"},
                {"op": aerospike.OPERATOR_INCR,
                 "bin": "age",
                 "val": 3}, {"op": aerospike.OPERATOR_READ,
                             "bin": "name"}]

        try:
            (key, meta, bins) = TestOperate.client.operate(key, list, meta, policy)

        except RecordGenerationError as exception:
            assert exception.code == 3L
        
        (key , meta, bins) = TestOperate.client.get(key)
        assert bins == { 'age' : 1, 'name': 'name1'}
        assert key == ('test', 'demo', None,
                bytearray(b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8'))

    def test_operate_with_policy_gen_GT_positive(self):
        """
        Invoke operate() with gen GT positive.
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'gen': aerospike.POLICY_GEN_GT
        }
        (key, meta) = TestOperate.client.exists(key)
        gen = meta['gen']
        meta = {'gen': gen + 5, 'ttl': 1200}

        list = [{"op": aerospike.OPERATOR_APPEND,
                 "bin": "name",
                 "val": "aa"},
                {"op": aerospike.OPERATOR_INCR,
                 "bin": "age",
                 "val": 3}, {"op": aerospike.OPERATOR_READ,
                             "bin": "name"}]

        (key, meta, bins) = TestOperate.client.operate(key, list, meta, policy)

        assert bins == {'name': 'name1aa'}
        assert key == ('test', 'demo', 1, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_opearte_with_incorrect_policy_negative(self):
        """
        Invoke operate() with incorrect policy
        """
        key = ('test', 'demo', 1)
        policy = {'timeout': 0.5}
        list = [
            {"op": aerospike.OPERATOR_PREPEND,
             "bin": "name",
             "val": "ram"},
            {"op": aerospike.OPERATOR_INCR,
             "bin": "age",
             "val": 3}, {"op": aerospike.OPERATOR_READ,
                         "bin": "name"}
        ]

        try:
            (bins) = TestOperate.client.operate(key, list, {}, policy)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "timeout is invalid"

    def test_opearte_on_same_bin_negative(self):
        """
        Invoke operate() on same bin
        """
        key = ('test', 'demo', 1)
        policy = {'timeout': 5000}
        list = [
            {"op": aerospike.OPERATOR_PREPEND,
             "bin": "name",
             "val": "ram"},
            {"op": aerospike.OPERATOR_APPEND,
             "bin": "name",
             "val": "aa"},
            {"op": aerospike.OPERATOR_INCR,
             "bin": "age",
             "val": 3}, {"op": aerospike.OPERATOR_READ,
                         "bin": "name"}
        ]

        try:
            (bins) = TestOperate.client.operate(key, list, {}, policy)

        except InvalidRequest as exception:
            assert exception.code == 4L

    def test_operate_with_nonexistent_key_positive(self):
        """
        Invoke operate() with non-existent key
        """
        key1 = ('test', 'demo', "key11")
        list = [
            {"op": aerospike.OPERATOR_PREPEND,
             "bin": "loc",
             "val": "mumbai"}, {"op": aerospike.OPERATOR_READ,
                                "bin": "loc"}
        ]
        key, meta, bins = TestOperate.client.operate(key1, list)

        assert bins == {'loc': 'mumbai'}
        TestOperate.client.remove(key1)

    def test_operate_with_nonexistent_bin_positive(self):
        """
        Invoke operate() with non-existent bin
        """
        key = ('test', 'demo', 1)
        list = [
            {"op": aerospike.OPERATOR_APPEND,
             "bin": "addr",
             "val": "pune"}, {"op": aerospike.OPERATOR_READ,
                              "bin": "addr"}
        ]
        key, meta, bins = TestOperate.client.operate(key, list)

        assert bins == {'addr': 'pune'}

    def test_operate_empty_string_key_negative(self):
        """
        Invoke operate() with empty string key
        """
        key = ('test', 'demo', 1)
        policy = {'timeout': 0.5}
        list = [
                {
                    "op" : aerospike.OPERATOR_PREPEND,
                    "bin" : "name",
                    "val" : "ram"
                    }
                ]
        try:
            TestOperate.client.operate("", list)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "key is invalid"

    def test_operate_with_extra_parameter_negative(self):
        """
        Invoke operate() with extra parameter.
        """
        key = ('test', 'demo', 1)
        policy = {'timeout': 1000}
        list = [
            {"op": aerospike.OPERATOR_PREPEND,
             "bin": "name",
             "val": "ram"}
        ]
        with pytest.raises(TypeError) as typeError:
            TestOperate.client.operate(key, list, {}, policy, "")

        assert "operate() takes at most 4 arguments (5 given)" in typeError.value

    def test_operate_policy_is_string_negative(self):
        """
        Invoke operate() with policy is string
        """
        key = ('test', 'demo', 1)
        list = [
                {
                    "op" : aerospike.OPERATOR_PREPEND,
                    "bin" : "name",
                    "val" : "ram"
                    }
                ]
        try:
            TestOperate.client.operate(key, list, {}, "")

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "policy must be a dict"

    def test_operate_key_is_none_negative(self):
        """
        Invoke operate() with key is none
        """
        list = [
                {
                    "op" : aerospike.OPERATOR_PREPEND,
                    "bin" : "name",
                    "val" : "ram"
                    }
                ]
        try:
            TestOperate.client.operate(None, list)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "key is invalid"

    def test_operate_append_withot_value_parameter_negative(self):
        """
        Invoke operate() with append operation and append val is not given
        """
        key = ('test', 'demo', 1)
        policy = {'timeout': 1000}

        list = [{"op": aerospike.OPERATOR_APPEND,
                 "bin": "name"},
                {"op": aerospike.OPERATOR_INCR,
                 "bin": "age",
                 "val": 3}]

        try:
            TestOperate.client.operate(key, list, {}, policy)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Value should be given"

    def test_operate_with_extra_parameter_negative(self):
        """
        Invoke operate() with more than 3 parameters given
        """
        key = ('test', 'demo', 1)
        policy = {'timeout': 1000}

        list = [{
            "op": aerospike.OPERATOR_APPEND,
            "bin": "name",
            "val": 3,
            "aa": 89
        }, ]

        try:
            TestOperate.client.operate(key, list, {}, policy)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "operation can contain only op, bin and val keys"

    def test_operate_append_value_integer_negative(self):
        """
        Invoke operate() with append value is of type integer
        """
        key = ('test', 'demo', 1)
        list = [{"op": aerospike.OPERATOR_APPEND,
                 "bin": "name",
                 "val": 12},
                {"op": aerospike.OPERATOR_INCR,
                 "bin": "age",
                 "val": 3}, {"op": aerospike.OPERATOR_READ,
                             "bin": "name"}]

        try:
            TestOperate.client.operate(key, list)
        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Cannot concatenate 'str' and 'non-str' objects"

    def test_operate_increment_nonexistent_key(self):
        """
        Invoke operate() with increment with nonexistent_key
        """
        key = ('test', 'demo', "non_existentkey")
        list = [{"op": aerospike.OPERATOR_INCR, "bin": "age", "val": 5}]

        TestOperate.client.operate(key, list)

        (key, meta, bins) = TestOperate.client.get(key)

        assert bins == {"age": 5}

        TestOperate.client.remove(key)

    def test_operate_increment_nonexistent_bin(self):
        """
        Invoke operate() with increment with nonexistent_bin
        """
        key = ('test', 'demo', 1)
        list = [{"op": aerospike.OPERATOR_INCR, "bin": "my_age", "val": 5}]

        TestOperate.client.operate(key, list)

        (key, meta, bins) = TestOperate.client.get(key)

        assert bins == {"my_age": 5, "age": 1, "name": "name1"}

    def test_operate_with_write_positive_float_value(self):
        """
        Invoke operate() with write operation float value
        """
        if TestOperate.skip_old_server == True:
            pytest.skip("Server does not support operation")
        key = ('test', 'demo', 1)
        list = [{
            "op": aerospike.OPERATOR_WRITE,
            "bin": "write_bin",
            "val": {"no": 89.8}
        }, {"op": aerospike.OPERATOR_READ,
            "bin": "write_bin"}]

        key, meta, bins = TestOperate.client.operate(key, list)

        assert bins == {'write_bin': {u'no': 89.8}}

    def test_operate_with_write_positive(self):
        """
        Invoke operate() with write operation
        """
        key = ('test', 'demo', 1)
        list = [{
            "op": aerospike.OPERATOR_WRITE,
            "bin": "write_bin",
            "val": {"no": 89}
        }, {"op": aerospike.OPERATOR_READ,
            "bin": "write_bin"}]

        key, meta, bins = TestOperate.client.operate(key, list)

        assert bins == {'write_bin': {u'no': 89}}

    def test_operate_with_write_tuple_positive(self):
        """
        Invoke operate() with write operation
        """
        key = ('test', 'demo', 1)
        list = [{
            "op": aerospike.OPERATOR_WRITE,
            "bin": "write_bin",
            "val": tuple('abc')
        }, {"op": aerospike.OPERATOR_READ,
            "bin": "write_bin"}]

        key, meta, bins = TestOperate.client.operate(key, list)

        assert bins == {'write_bin': ('a', 'b', 'c')}

    def test_operate_with_correct_paramters_positive_without_connection(self):
        """
        Invoke operate() with correct parameters without connection
        """
        key = ('test', 'demo', 1)
        config = {'hosts': [('127.0.0.1', 3000)]}
        client1 = aerospike.client(config)
        list = [
            {"op": aerospike.OPERATOR_PREPEND,
             "bin": "name",
             "val": u"ram"},
            {"op": aerospike.OPERATOR_INCR,
             "bin": "age",
             "val": 3}, {"op": aerospike.OPERATOR_READ,
                         "bin": "name"}
        ]

        try:
            key, meta, bins = client1.operate(key, list)

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'

    def test_operate_with_incr_value_string(self):
        """
        Invoke operate() with incr value negative
        """
        try:
            key = ('test', 'demo', 1)
            policy = {
                'timeout': 1000,
                'key': aerospike.POLICY_KEY_SEND,
                'commit_level': aerospike.POLICY_COMMIT_LEVEL_MASTER
            }

            list = [ {"op": aerospike.OPERATOR_INCR,
                     "bin": "age",
                     "val": "3"}, {"op": aerospike.OPERATOR_READ,
                                 "bin": "age"}]

            key, meta, bins = TestOperate.client.operate(key, list, {}, policy)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Unsupported operand type(s) for +: only 'int' allowed"

        TestOperate.client.remove(key)

    def test_operate_with_bin_bytearray_positive(self):
        """
        Invoke operate() with correct parameters
        """
        key = ('test', 'demo', 1)
        list = [
            {"op": aerospike.OPERATOR_PREPEND,
             "bin": bytearray("asd[;asjk", "utf-8"),
             "val": u"ram"},
            {"op": aerospike.OPERATOR_READ,
                "bin": bytearray("asd[;asjk", "utf-8")}
        ]

        key, meta, bins = TestOperate.client.operate(key, list)

        assert bins == {'asd[;asjk': 'ram'}

    def test_operate_with_operatorappend_valbytearray(self):
        """
        Invoke operate() with operator as append and value is a bytearray
        """
        key = ('test', 'demo', 'bytearray_key')
        list = [
            {"op": aerospike.OPERATOR_APPEND,
             "bin": "bytearray_bin",
             "val": bytearray("abc")},
            {"op": aerospike.OPERATOR_READ,
             "bin": "bytearray_bin"}
        ]

        key, meta, bins = TestOperate.client.operate(key, list)

        assert bins == {'bytearray_bin': "asd;as[d'as;dabc"}

    def test_operate_with_operatorappend_valbytearray_newrecord(self):
        """
        Invoke operate() with operator as append and value is a bytearray and a
        new record(does not exist)
        """
        key = ('test', 'demo', 'bytearray_new')
        list = [
            {"op": aerospike.OPERATOR_APPEND,
             "bin": "bytearray_bin",
             "val": bytearray("asd;as[d'as;d", "utf-8")},
            {"op": aerospike.OPERATOR_READ,
             "bin": "bytearray_bin"}
        ]

        key, meta, bins = TestOperate.client.operate(key, list)

        assert bins == {'bytearray_bin': "asd;as[d'as;d"}

        TestOperate.client.remove(key)

    def test_operate_with_operatorprepend_valbytearray(self):
        """
        Invoke operate() with operator as prepend and value is a bytearray
        """
        key = ('test', 'demo', 'bytearray_key')
        list = [
            {"op": aerospike.OPERATOR_PREPEND,
             "bin": "bytearray_bin",
             "val": bytearray("abc")},
            {"op": aerospike.OPERATOR_READ,
             "bin": "bytearray_bin"}
        ]

        key, meta, bins = TestOperate.client.operate(key, list)

        assert bins == {'bytearray_bin': "abcasd;as[d'as;d"}

    def test_operate_with_operatorprepend_valbytearray_newrecord(self):
        """
        Invoke operate() with operator as prepend and value is a bytearray and a
        new record(does not exist)
        """
        key = ('test', 'demo', 'bytearray_new')
        list = [
            {"op": aerospike.OPERATOR_PREPEND,
             "bin": "bytearray_bin",
             "val": bytearray("asd;as[d'as;d", "utf-8")},
            {"op": aerospike.OPERATOR_READ,
             "bin": "bytearray_bin"}
        ]

        key, meta, bins = TestOperate.client.operate(key, list)

        assert bins == {'bytearray_bin': "asd;as[d'as;d"}

        TestOperate.client.remove(key)

    def test_operate_write_set_to_aerospike_null(self):
		"""
        	Invoke operate() with write command with bin set to aerospike_null
		"""
		key = ('test', 'demo', 'null_record')

		bins = {"name": "John", "no": 3}

		assert 0 == TestOperate.client.put(key, bins)

		(key, meta, bins) = TestOperate.client.get(key)

		assert {"name": "John", "no": 3} == bins

		list = [
			{
				"op" : aerospike.OPERATOR_WRITE,
				"bin": "no",
				"val": aerospike.null
			},
			{
				"op": aerospike.OPERATOR_READ,
				"bin": "no"
			}
        ]	

		(key, meta, bins) = TestOperate.client.operate(key, list)

		assert {} == bins

		TestOperate.client.remove(key)

    def test_operate_prepend_with_int_new_record(self):
        """
            Invoke operate() with prepend command on a new record
	    """
        key = ('test', 'demo', 'prepend_int')

        list = [
            {
                "op" : aerospike.OPERATOR_PREPEND,
                "bin": "age",
                "val": 4
            },
            {
                "op": aerospike.OPERATOR_READ,
                "bin": "age"
            }
        ]	

        (key, meta, bins) = TestOperate.client_no_typechecks.operate(key, list)

        assert {'age': 4} == bins

        TestOperate.client_no_typechecks.remove(key)

    def test_operate_prepend_with_int_existing_record(self):
        """
            Invoke operate() with prepend command on a existing record
        """
        key = ('test', 'demo', 1)

        list = [
            {
                "op" : aerospike.OPERATOR_PREPEND,
                "bin": "age",
                "val": 4
            },
            {
                "op": aerospike.OPERATOR_READ,
                "bin": "age"
            }
        ]	

        try:
            (key, meta, bins) = TestOperate.client_no_typechecks.operate(key, list)

        except BinIncompatibleType as exception:
            assert exception.code == 12L

        TestOperate.client_no_typechecks.remove(key)

    def test_operate_prepend_with_list_existing_record(self):
        """
            Invoke operate() with prepend command on a existing record
        """
        key = ('test', 'demo', 'existing_key')

        (key, old_meta) = TestOperate.client_no_typechecks.exists(key)

        list = [
            {
                "op" : aerospike.OPERATOR_PREPEND,
                "bin": "list",
                "val": ['c']
            },
            {
                "op": aerospike.OPERATOR_READ,
                "bin": "list"
            }
        ]	

        exception_raised = False
        try:
            (key, meta, bins) = TestOperate.client_no_typechecks.operate(key, list)
        except BinIncompatibleType as exception:
            assert exception.code == 12L
            exception_raised = True
        assert exception_raised is True


        TestOperate.client_no_typechecks.remove(key)

    def test_operate_append_with_dict_new_record(self):
        """
            Invoke operate() with append command on a new record
	    """
        key = ('test', 'demo', 'append_dict')

        list = [
            {
                "op" : aerospike.OPERATOR_APPEND,
                "bin": "dict",
                "val": {"a": 1, "b": 2}
            },
            {
                "op": aerospike.OPERATOR_READ,
                "bin": "dict"
            }
        ]	

        (key, meta, bins) = TestOperate.client_no_typechecks.operate(key, list)

        assert {'dict': {"a": 1, "b": 2}} == bins

        TestOperate.client_no_typechecks.remove(key)

    def test_operate_append_with_dict_existing_record(self):
        """
            Invoke operate() with append command on a existing record
        """
        key = ('test', 'demo', 'existing_key')

        (key, old_meta) = TestOperate.client_no_typechecks.exists(key)

        list = [
            {
                "op" : aerospike.OPERATOR_APPEND,
                "bin": "dict",
                "val": {"c": 2}
            },
            {
                "op": aerospike.OPERATOR_READ,
                "bin": "dict"
            }
        ]

        exception_raised = False
        try:
            (key, meta, bins) = TestOperate.client_no_typechecks.operate(key, list)
        except BinIncompatibleType as exception:
            assert exception.code == 12L
            exception_raised = True
        assert exception_raised is True

        TestOperate.client_no_typechecks.remove(key)

    def test_operate_append_with_float_existing_record(self):
        """
            Invoke operate() with append command on a existing record
        """
        key = ('test', 'demo', 'existing_key')

        list = [
            {
                "op" : aerospike.OPERATOR_APPEND,
                "bin": "float",
                "val": 3.4
            },
            {
                "op": aerospike.OPERATOR_READ,
                "bin": "float"
            }
        ]	

        try:
            (key, meta, bins) = TestOperate.client_no_typechecks.operate(key, list)

        except BinIncompatibleType as exception:
            assert exception.code == 12L

        TestOperate.client_no_typechecks.remove(key)

    def test_operate_incr_with_string_new_record(self):
        """
            Invoke operate() with incr command on a new record
	    """
        key = ('test', 'demo', 'incr_string')

        list = [
            {
                "op" : aerospike.OPERATOR_INCR,
                "bin": "name",
                "val": "aerospike"
            },
            {
                "op": aerospike.OPERATOR_READ,
                "bin": "name"
            }
        ]	

        (key, meta, bins) = TestOperate.client_no_typechecks.operate(key, list)

        assert {'name': 'aerospike'} == bins

        TestOperate.client_no_typechecks.remove(key)

    def test_operate_incr_with_string_existing_record(self):
        """
            Invoke operate() with incr command on a existing record
        """
        key = ('test', 'demo', 1)

        list = [
            {
                "op" : aerospike.OPERATOR_INCR,
                "bin": "name",
                "val": "aerospike"
            },
            {
                "op": aerospike.OPERATOR_READ,
                "bin": "name"
            }
        ]	

        try:
            (key, meta, bins) = TestOperate.client_no_typechecks.operate(key, list)

        except BinIncompatibleType as exception:
            assert exception.code == 12L

        TestOperate.client_no_typechecks.remove(key)

    def test_operate_incr_with_bytearray_existing_record(self):
        """
            Invoke operate() with incr command on a new record
	    """
        key = ('test', 'demo', 'existing_key')

        list = [
            {
                "op" : aerospike.OPERATOR_INCR,
                "bin": "bytearray",
                "val": bytearray("abc")
            },
            {
                "op": aerospike.OPERATOR_READ,
                "bin": "bytearray"
            }
        ]	

        try:
            (key, meta, bins) = TestOperate.client_no_typechecks.operate(key, list)

        except BinIncompatibleType as exception:
            assert exception.code == 12L

        TestOperate.client_no_typechecks.remove(key)

    def test_operate_incr_with_geospatial_new_record(self):
        """
            Invoke operate() with incr command on a new record
	    """
        key = ('test', 'demo', 'geospatial_key')

        list = [
            {
                "op" : aerospike.OPERATOR_INCR,
                "bin": "geospatial",
                "val": aerospike.GeoJSON({"type": "Point", "coordinates":
                    [42.34, 58.62] })
            },
            {
                "op": aerospike.OPERATOR_READ,
                "bin": "geospatial"
            }
        ]	

        (key, meta, bins) = TestOperate.client_no_typechecks.operate(key, list)

        assert bins == {'geospatial': {'coordinates': [42.34, 58.62], 'type':
            'Point'}}
        TestOperate.client_no_typechecks.remove(key)

    def test_operate_with_bin_length_extra(self):
        """
        Invoke operate() with bin length extra. Strict types enabled
        """
        key = ('test', 'demo', 1)

        max_length = 'a'
        for i in xrange(20):
            max_length = max_length + 'a'

        list = [
            {"op": aerospike.OPERATOR_PREPEND,
             "bin": "name",
             "val": u"ram"},
            {"op": aerospike.OPERATOR_INCR,
             "bin": max_length,
             "val": 3}, {"op": aerospike.OPERATOR_READ,
                         "bin": "name"}
        ]

        try:
            key, meta, bins = TestOperate.client.operate(key, list)

        except BinNameError as exception:
            assert exception.code == 21L

    def test_operate_with_bin_length_extra_nostricttypes(self):
        """
        Invoke operate() with bin length extra. Strict types disabled
        """
        key = ('test', 'demo', 1)

        max_length = 'a'
        for i in xrange(20):
            max_length = max_length + 'a'

        list = [
            {"op": aerospike.OPERATOR_PREPEND,
             "bin": "name",
             "val": u"ram"},
            {"op": aerospike.OPERATOR_INCR,
             "bin": max_length,
             "val": 3}
        ]

        TestOperate.client_no_typechecks.operate(key, list)

        (key, meta, bins) = TestOperate.client_no_typechecks.get(key)

        assert bins == {"name": "ramname1", "age": 1}

    def test_operate_with_command_invalid(self):
        """
        Invoke operate() with an invalid command. Strict types enabled
        """
        key = ('test', 'demo', 1)

        list = [
            {"op": aerospike.OPERATOR_PREPEND,
             "bin": "name",
             "val": u"ram"},
            {"op": 3,
             "bin": "age",
             "val": 3}, {"op": aerospike.OPERATOR_READ,
                         "bin": "name"}
        ]

        try:
            key, meta, bins = TestOperate.client.operate(key, list)

        except ParamError as exception:
            assert exception.code == -2L

    def test_operate_with_command_invalid_nostricttypes(self):
        """
        Invoke operate() with an invalid command. Strict types disabled
        """
        key = ('test', 'demo', 1)

        list = [
            {"op": aerospike.OPERATOR_PREPEND,
             "bin": "name",
             "val": u"ram"},
            {"op": 3,
             "bin": "age",
             "val": 3}, {"op": aerospike.OPERATOR_READ,
                         "bin": "name"}
        ]

        key, meta, bins = TestOperate.client_no_typechecks.operate(key, list)

        assert bins == {'name': 'ramname1'}

    def test_operate_prepend_set_to_aerospike_null(self):
        """
            Invoke operate() with prepend command with bin set to aerospike_null
        """
        key = ('test', 'demo', 'null_record')

        bins = {"name": "John", "no": 3}

        assert 0 == TestOperate.client.put(key, bins)

        (key, meta, bins) = TestOperate.client.get(key)

        assert {"name": "John", "no": 3} == bins

        list = [
            {
                "op" : aerospike.OPERATOR_PREPEND,
                "bin": "no",
                "val": aerospike.null
            },
            {
                "op": aerospike.OPERATOR_READ,
                "bin": "no"
            }
        ]	

        try:
            (key, meta, bins) = TestOperate.client.operate(key, list)

        except InvalidRequest as exception:
            assert exception.code == 4
        TestOperate.client.remove(key)



# -*- coding: utf-8 -*-

import pytest
import sys
import cPickle as pickle
from test_base_class import TestBaseClass
import os
import shutil
import time

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

from aerospike import predicates as p

class TestAggregate(TestBaseClass):
    def setup_class(cls):
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {
            'hosts': hostlist,
            'lua':{'user_path': '/tmp/',
            'system_path':'../aerospike-client-c/lua/'}}
        if user == None and password == None:
            client = aerospike.client(config).connect()
        else:
            client = aerospike.client(config).connect(user, password)

        TestAggregate.skip_old_server = True
        versioninfo = client.info('version')
        for keys in versioninfo:
            for value in versioninfo[keys]:
                if value != None:
                    versionlist = value[value.find("build") + 6:value.find("\n")].split(".")
                    if int(versionlist[0]) >= 3 and int(versionlist[1]) >= 6:
                        TestAggregate.skip_old_server = False
        client.index_integer_create('test', 'demo', 'test_age',
                'test_demo_test_age_idx')
        client.index_integer_create('test', 'demo', 'age1', 'test_demo_age1_idx')
        time.sleep(2)

        filename = "stream_example.lua"
        udf_type = aerospike.UDF_TYPE_LUA
        status = client.udf_put(filename, udf_type)
        shutil.copyfile(filename, config['lua']['user_path'] +
            'stream_example.lua')
        client.close()

    def teardown_class(cls):
        return
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {
            'hosts': hostlist,
            'lua':{'user_path': '/tmp/',
            'system_path':'../aerospike-client-c/lua/'}}
        if user == None and password == None:
            client = aerospike.client(config).connect()
        else:
            client = aerospike.client(config).connect(user, password)
        client.index_remove('test', 'test_demo_test_age_idx')
        client.index_remove('test', 'test_demo_age1_idx')
        module = "stream_example.lua"

        status = client.udf_remove(module)
        os.remove(config['lua']['user_path'] + 'stream_example.lua')
        client.close()

    def setup_method(self, method):
        """
        Setup method.
        """

        hostlist, user, password = TestBaseClass.get_hosts()
        config = {
            'hosts': hostlist,
            'lua':{'user_path': '/tmp/',
            'system_path':'../aerospike-client-c/lua/'}}
        if TestBaseClass.user == None and TestBaseClass.password == None:
            self.client = aerospike.client(config).connect()
        else:
            self.client = aerospike.client(config).connect(
                TestBaseClass.user, TestBaseClass.password)

        for i in xrange(5):
            key = ('test', 'demo', i)
            rec = {
                'name': 'name%s' % (str(i)),
                'addr': 'name%s' % (str(i)),
                'test_age': i,
                'no': i
            }
            self.client.put(key, rec)

    def teardown_method(self, method):
        """
        Teardown method.
        """
        for i in xrange(5):
            key = ('test', 'demo', i)
            #self.client.remove(key)
        self.client.close()

    def test_aggregate_with_no_parameters(self):
        """
            Invoke aggregate() without any mandatory parameters.
        """
        try:
            query = self.client.query()
            query.select()
            query.where()

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'query() expects atleast 1 parameter'

        #assert "where() takes at least 1 argument (0 given)" in typeError.value

    def test_aggregate_no_sec_index(self):
        """
            Invoke aggregate() with no secondary index
        """
        try:
            query = self.client.query('test', 'demo')
            query.select('name', 'no')
            query.where(p.between('no', 1, 5))
            query.apply('stream_example', 'count')

            result = None

            def user_callback(value):
                result = value

            query.foreach(user_callback)
        except IndexNotFound as exception:
            assert exception.code == 201L
            assert exception.msg == 'AEROSPIKE_ERR_INDEX_NOT_FOUND'

    def test_aggregate_with_incorrect_ns_set(self):
        """
            Invoke aggregate() with incorrect ns and set
        """
        try:
            query = self.client.query('test1', 'demo1')
            query.select('name', 'test_age')
            query.where(p.equals('test_age', 1))
            query.apply('stream_example', 'count')
            result = 1

            def user_callback(value):
                result = value

            query.foreach(user_callback)

        except InvalidRequest as exception:
            assert exception.code == 4L
            assert exception.msg == 'AEROSPIKE_ERR_REQUEST_INVALID'

    #@pytest.mark.xfail(reason="C client incorrectly sent status AEROSPIKE_ERR_UDF")
    def test_aggregate_with_where_incorrect(self):
        """
            Invoke aggregate() with where is incorrect
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.equals('test_age', 165))
        query.apply('stream_example', 'count')
        records = []

        def user_callback(value):
            records.append(value)

        query.foreach(user_callback)
        assert records == []

    def test_aggregate_with_where_none_value(self):
        """
            Invoke aggregate() with where is null value
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        try:
            query.where(p.equals('test_age', None))
            query.apply('stream_example', 'count')
            result = 1

            def user_callback(value):
                result = value

            query.foreach(user_callback)

        except ParamError as exception:
            assert exception.code == -2L
            assert exception.msg == 'predicate is invalid.'

    #@pytest.mark.xfail(reason="C client incorrectly sent status AEROSPIKE_ERR_UDF")
    def test_aggregate_with_where_bool_value(self):
        """
            Invoke aggregate() with where is bool value
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.between('test_age', True, True))
        query.apply('stream_example', 'count')
        records = []

        def user_callback(value):
            records.append(value)

        query.foreach(user_callback)
        assert records[0] == 1

    def test_aggregate_with_where_equals_value(self):
        """
            Invoke aggregate() with where is equal
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.equals('test_age', 2))
        query.apply('stream_example', 'count')
        records = []

        def user_callback(value):
            records.append(value)

        query.foreach(user_callback)
        assert records[0] == 1

    def test_aggregate_with_empty_module_function(self):
        """
            Invoke aggregate() with empty module and function
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.between('test_age', 1, 5))
        query.apply('', '')

        result = None

        def user_callback(value):
            result = value

        query.foreach(user_callback)
        assert result == None
    """
    def test_aggregate_with_incorrect_module(self):
            #Invoke aggregate() with incorrect module
        try:
            query = self.client.query('test', 'demo')
            query.select('name', 'test_age')
            query.where(p.between('test_age', 1, 5))
            query.apply('streamwrong', 'count')

            result = None

            def user_callback(value):
                result = value

            query.foreach(user_callback)

        except ClientError as exception:
            assert exception.code == -1L
            assert exception.msg == 'UDF: Execution Error 1'

    def test_aggregate_with_incorrect_function(self):
        #Invoke aggregate() with incorrect function
        try:
            query = self.client.query('test', 'demo')
            query.select('name', 'test_age')
            query.where(p.between('test_age', 1, 5))
            query.apply('stream_example', 'countno')

            records = []

            def user_callback(value):
                records.append(value)

            query.foreach(user_callback)
        except ClientError as exception:
            assert exception.code == -1L
            assert exception.msg == 'UDF: Execution Error 2 : function not found'
            """
    def test_aggregate_with_correct_parameters(self):
        """
            Invoke aggregate() with correct arguments
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.between('test_age', 1, 5))
        query.apply('stream_example', 'count')

        records = []

        def user_callback(value):
            records.append(value)

        query.foreach(user_callback)
        assert records[0] == 4

    def test_aggregate_with_policy(self):
        """
            Invoke aggregate() with policy
        """
        policy = {'timeout': 1000}
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.between('test_age', 1, 5))
        query.apply('stream_example', 'count')

        records = []

        def user_callback(value):
            records.append(value)

        query.foreach(user_callback, policy)
        assert records[0] == 4

    def test_aggregate_with_extra_parameter(self):
        """
            Invoke aggregate() with extra parameter
        """
        policy = {'timeout': 1000}

        with pytest.raises(TypeError) as typeError:
            query = self.client.query('test', 'demo')
            query.select('name', 'test_age')
            query.where(p.between('test_age', 1, 5))
            query.apply('stream_example', 'count')

            result = None

            def user_callback(value):
                result = value

            query.foreach(user_callback, policy, "")

        assert "foreach() takes at most 2 arguments (3 given)" in typeError.value

    def test_aggregate_with_extra_parameters_to_lua(self):
        """
            Invoke aggregate() with extra arguments
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.between('test_age', 1, 5))
        stream = None
        query.apply('stream_example', 'count', [stream])

        records = []

        def user_callback(value):
            records.append(value)

        query.foreach(user_callback)
        assert records[0] == 4

    def test_aggregate_with_extra_parameter_in_lua(self):
        """
            Invoke aggregate() with extra parameter in lua
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.between('test_age', 1, 5))
        query.apply('stream_example', 'count_extra')

        records = []

        def user_callback(value):
            records.append(value)

        query.foreach(user_callback)
        assert records[0] == 4

    """
    def test_aggregate_with_less_parameter_in_lua(self):
        #Invoke aggregate() with less parameter in lua
        try:
            query = self.client.query('test', 'demo')
            query.select('name', 'test_age')
            query.where(p.between('test_age', 1, 5))
            query.apply('stream_example', 'count_less')

            records = []

            def user_callback(value):
                records.append(value)

            query.foreach(user_callback)

        except ClientError as exception:
            assert exception.code == -1L
            """
    def test_aggregate_with_arguments_to_lua_function(self):
        """
            Invoke aggregate() with unicode arguments to lua function.
        """
        query = self.client.query('test', 'demo')
        query.where(p.between('test_age', 0, 5))
        query.apply('stream_example', 'group_count', [u"name", u"addr"])

        rec = []

        def callback(value):
            rec.append(value)

        query.foreach(callback)
        assert rec == [
            {u'name4': 1,
             u'name2': 1,
             u'name3': 1,
             u'name0': 1,
             u'name1': 1}
        ]

    def test_aggregate_with_arguments_to_lua_function_having_float_value(self):
        """
            Invoke aggregate() with unicode arguments to lua function having a
            float value
        """
        if TestAggregate.skip_old_server == True:
            pytest.skip("Server does not support aggregate on float type as lua argument")
        query = self.client.query('test', 'demo')
        query.where(p.between('test_age', 0, 5))
        query.apply('stream_example', 'double_group_count', [u"name", u"addr", 2.5])

        rec = []

        def callback(value):
            rec.append(value)

        query.foreach(callback)
        assert rec == [
            {u'name4': 3.5,
             u'name2': 3.5,
             u'name3': 3.5,
             u'name0': 3.5,
             u'name1': 3.5}
        ]

    def test_aggregate_with_unicode_module_and_function_name(self):
        """
            Invoke aggregate() with unicode module and function names
        """
        query = self.client.query('test', 'demo')
        query.select(u'name', 'test_age')
        query.where(p.between('test_age', 1, 5))
        query.apply(u'stream_example', u'count')

        records = []

        def user_callback(value):
            records.append(value)

        query.foreach(user_callback)
        assert records[0] == 4

    def test_aggregate_with_multiple_foreach_on_same_query_object(self):
        """
            Invoke aggregate() with multiple foreach on same query object.
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.between('test_age', 1, 5))
        query.apply('stream_example', 'count')

        records = []

        def user_callback(value):
            records.append(value)

        query.foreach(user_callback)
        assert records[0] == 4

        records = []
        query.foreach(user_callback)
        assert records[0] == 4

    def test_aggregate_with_multiple_results_call_on_same_query_object(self):
        """
            Invoke aggregate() with multiple foreach on same query object.
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.between('test_age', 1, 5))
        query.apply('stream_example', 'count')

        records = []
        records = query.results()
        assert records[0] == 4

        records = []
        records = query.results()
        assert records[0] == 4

    def test_aggregate_with_correct_parameters_without_connection(self):
        """
            Invoke aggregate() with correct arguments without connection
        """
        config = {'hosts': [('127.0.0.1', 3000)]}
        client1 = aerospike.client(config)

        try:
            query = client1.query('test', 'demo')
            query.select('name', 'test_age')
            query.where(p.between('test_age', 1, 5))
            query.apply('stream_example', 'count')

            records = []

            def user_callback(value):
                records.append(value)

            query.foreach(user_callback)

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'

    def test_aggregate_with_correct_parameters_lua_file_not_in_same_dir(self):
        """
            Invoke aggregate() with correct arguments and lua file not in same
            directory
        """
        os.remove('stream_example.lua')
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.between('test_age', 1, 5))
        query.apply('stream_example', 'count')

        records = []

        def user_callback(value):
            records.append(value)

        query.foreach(user_callback)
        assert records[0] == 4

        assert True if os.path.isfile('/tmp/stream_example.lua') else False

        shutil.move('/tmp/stream_example.lua', '.')



# -*- coding: utf-8 -*-

import pytest
import sys
import cPickle as pickle
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestClose(TestBaseClass):
    def setup_class(cls):
        hostlist, user, password = TestBaseClass.get_hosts()

    def test_close_positive(self):
        """
            Invoke close() after positive connect
        """
        config = {'hosts': TestClose.hostlist}
        if TestClose.user == None and TestClose.password == None:
            self.client = aerospike.client(config).connect()
        else:
            self.client = aerospike.client(config).connect(TestClose.user,
                                                           TestClose.password)

        self.closeobject = self.client.close()
        assert self.closeobject == None

    def test_close_negative(self):
        """
            Invoke close() after negative connect
        """
        config = {'hosts': [('127.0.0.1', 2000)]}

        with pytest.raises(Exception) as exception:
            self.client = aerospike.client(config).connect()
        with pytest.raises(AttributeError) as attributeError:
            self.closeobject = self.client.close()
        assert "'TestClose' object has no attribute 'client'" in attributeError.value

    def test_close_positive_without_connection(self):
        """
            Invoke close() without connection
        """
        config = {'hosts': [('127.0.0.1', 3000)]}
        self.client = aerospike.client(config)

        try:
            self.closeobject = self.client.close()

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'



# -*- coding: utf-8 -*-

import pytest
import sys
import time
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestQueryUsers(TestBaseClass):

    pytestmark = pytest.mark.skipif(
        TestBaseClass().get_hosts()[1] == None,
        reason="No user specified, may be not secured cluster.")

    def setup_method(self, method):
        """
        Setup method
        """
        hostlist, user, password = TestBaseClass().get_hosts()
        config = {"hosts": hostlist}
        TestQueryUsers.Me = self
        self.client = aerospike.client(config).connect(user, password)
        
        try:
            self.client.admin_drop_user("example-test")
        except:
            pass
        policy = {}
        user = "example-test"
        password = "foo2"
        roles = ["read-write", "sys-admin", "read"]

        status = self.client.admin_create_user( user, password, roles, policy )

        self.delete_users = []

    def teardown_method(self, method):
        """
        Teardown method
        """

        policy = {}

        self.client.admin_drop_user( "example-test", policy )

        self.client.close()

    def test_query_users_with_proper_parameters(self):

        policy = {}

        time.sleep(2)
        user_details = self.client.admin_query_users()

        assert user_details['example-test'] == ['read', 'read-write', 'sys-admin']

    def test_query_users_with_invalid_timeout_policy_value(self):

        policy = {"timeout": 0.1}
        user = "example-test"

        try:
            status = self.client.admin_query_users( policy )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "timeout is invalid"

    def test_query_users_with_proper_timeout_policy_value(self):

        policy = {'timeout': 50}

        time.sleep(2)
        user_details = self.client.admin_query_users(policy)

        time.sleep(2)
        assert user_details['example-test'] == ['read', 'read-write', 'sys-admin']

    def test_query_users_with_no_roles(self):

        policy = {}
        user = "example-test"
        roles = ["sys-admin", "read", "read-write"]

        status = self.client.admin_revoke_roles(user, roles, policy)
        assert status == 0
        time.sleep(2)

        user_details = self.client.admin_query_users(policy)

        time.sleep(2)
        assert user_details['example-test'] == []

    def test_query_users_with_extra_argument(self):
        """
            Invoke query_users() with extra argument.
        """
        policy = {'timeout': 1000}
        with pytest.raises(TypeError) as typeError:
            self.client.admin_query_users(policy, "")

        assert "admin_query_users() takes at most 1 argument (2 given)" in typeError.value

    def test_query_users_with_policy_as_string(self):
        """
            Invoke query_users() with policy as string
        """
        policy = ""
        try:
            self.client.admin_query_users( policy )

        except ParamError as exception:
            assert exception.code == -2L
            assert exception.msg == "policy must be a dict"



# -*- coding: utf-8 -*-

import pytest
import sys
import time
from test_base_class import TestBaseClass
from types import *

aerospike = pytest.importorskip("aerospike")
try:
    import aerospike
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestUdfList(TestBaseClass):
    def setup_class(cls):
        """
        Setup class
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}

        if user == None and password == None:
            TestUdfList.client = aerospike.client(config).connect()
        else:
            TestUdfList.client = aerospike.client(config).connect(user,
                                                                  password)

        TestUdfList.client.udf_put('example.lua', 0, {})

    def teardown_class(cls):
        """
        Teardown class
        """
        TestUdfList.client.udf_remove("example.lua")

        TestUdfList.client.close()

    def test_udf_list_without_any_paramter(self):

        ulist = TestUdfList.client.udf_list()
        assert type(ulist) is ListType

    def test_udf_list_with_proper_parameters(self):

        policy = {'timeout': 0}
        udf_list = TestUdfList.client.udf_list(policy)

        present = False
        for udf in udf_list:
            if 'example.lua' == udf['name']:
                present = True

        assert True if present else False

    def test_udf_list_with_timeout_policy(self):

        policy = {'timeout': 1000}

        udf_list = TestUdfList.client.udf_list(policy)

        present = False
        for udf in udf_list:
            if 'example.lua' == udf['name']:
                present = True

        assert True if present else False

    def test_udf_list_with_invalid_timeout_policy_value(self):

        policy = {'timeout': 0.1}

        try:
            udf_list = TestUdfList.client.udf_list( policy )

        except ParamError as exception:
            assert exception.code == -2

            assert exception.msg == 'timeout is invalid'

    def test_udf_list_with_proper_parameters_without_connection(self):

        config = {'hosts': [('127.0.0.1', 3000)]}

        client1 = aerospike.client(config)

        policy = {'timeout': 0}

        try:
            udf_list = client1.udf_list( policy )

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'



# -*- coding: utf-8 -*-
################################################################################
# Copyright 2013-2015 Aerospike, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
################################################################################

from __future__ import print_function
import errno
import os
import platform
import sys
from distutils.command.build import build
from setuptools.command.install import install
from setuptools import setup, Extension
from shutil import copytree, copy2
from subprocess import call


class InstallCommand(install):
    user_options = install.user_options + [
        ('lua-system-path=', None, 'Path to the lua system files')
    ]

    def initialize_options(self):
        install.initialize_options(self)
        self.lua_system_path = None

    def finalize_options(self):
        install.finalize_options(self)

    def run(self):
        global lua_system_path
        lua_system_path = self.lua_system_path
        install.run(self)

class BuildCommand(build):
    user_options = build.user_options + [
        ('lua-system-path=', None, 'Path to the lua system files')
    ]

    def initialize_options(self):
        build.initialize_options(self)
        self.lua_system_path = None

    def finalize_options(self):
        build.finalize_options(self)

    def run(self):
        global lua_system_path
        lua_system_path = self.lua_system_path
        build.run(self)


################################################################################
# ENVIRONMENT VARIABLES
################################################################################

os.putenv('ARCHFLAGS','-arch x86_64')
os.environ['ARCHFLAGS'] = '-arch x86_64'
AEROSPIKE_C_VERSION = os.getenv('AEROSPIKE_C_VERSION')
if not AEROSPIKE_C_VERSION:
    AEROSPIKE_C_VERSION = '3.1.24'
DOWNLOAD_C_CLIENT = os.getenv('DOWNLOAD_C_CLIENT')
AEROSPIKE_C_HOME = os.getenv('AEROSPIKE_C_HOME')
PREFIX = None
PLATFORM =  platform.platform(1)
LINUX = 'Linux' in PLATFORM
DARWIN = 'Darwin' in PLATFORM
CWD = os.path.abspath(os.path.dirname(__file__))

################################################################################
# HELPER FUNCTION FOR RESOLVING THE C CLIENT DEPENDENCY
################################################################################
def lua_syspath_error(lua_system_path, exit_code):
    print("error: need permission to copy the Lua system files to ",
          lua_system_path, "or change the --lua-system-path")
    sys.exit(exit_code)

def resolve_c_client(lua_src_path, lua_system_path):
    global PREFIX, AEROSPIKE_C_VERSION, DOWNLOAD_C_CLIENT
    global extra_objects, include_dirs

    if PREFIX:
        os.putenv('PREFIX', PREFIX)
    if AEROSPIKE_C_VERSION:
        os.putenv('AEROSPIKE_C_VERSION', AEROSPIKE_C_VERSION)
    if DOWNLOAD_C_CLIENT:
        os.putenv('DOWNLOAD_C_CLIENT', DOWNLOAD_C_CLIENT)

    print('info: Executing','./scripts/aerospike-client-c.sh', file=sys.stdout)
    os.chmod('./scripts/aerospike-client-c.sh',0755)
    rc = call(['./scripts/aerospike-client-c.sh'])
    if rc != 0 :
        print("error: scripts/aerospike-client-c.sh", rc, file=sys.stderr)
        sys.exit(1)

    # Prefix for Aerospike C client libraries and headers
    aerospike_c_prefix = './aerospike-client-c'
    if not os.path.isdir(aerospike_c_prefix):
        print("error: Directory not found:", aerospike_c_prefix, file=sys.stderr)
        sys.exit(2)

    #-------------------------------------------------------------------------------
    # Check for aerospike.h
    #-------------------------------------------------------------------------------
    aerospike_h = aerospike_c_prefix + '/include/aerospike/aerospike.h'
    if not os.path.isfile(aerospike_h):
        print("error: aerospike.h not found:", aerospike_h, file=sys.stderr)
        sys.exit(3)
    print("info: aerospike.h found:", aerospike_h, file=sys.stdout)
    include_dirs = include_dirs + [
        aerospike_c_prefix + '/include', 
        aerospike_c_prefix + '/include/ck'
        ]

    #-------------------------------------------------------------------------------
    # Check for libaerospike.a
    #-------------------------------------------------------------------------------
    aerospike_a = aerospike_c_prefix + '/lib/libaerospike.a'
    if not os.path.isfile(aerospike_a):
        print("error: libaerospike.a not found:", aerospike_a, file=sys.stderr)
        sys.exit(4)
    print("info: libaerospike.a found:", aerospike_a, file=sys.stdout)
    extra_objects = extra_objects + [
        aerospike_a
        ]

    #---------------------------------------------------------------------------
    # Environment Variables
    #---------------------------------------------------------------------------
    os.putenv('CPATH', ':'.join(include_dirs))
    os.putenv('LD_LIBRARY_PATH', ':'.join(library_dirs))
    os.putenv('DYLD_LIBRARY_PATH', ':'.join(library_dirs))

    #---------------------------------------------------------------------------
    # Deploying the system lua files
    #---------------------------------------------------------------------------
    print("copying from", lua_src_path, "to", lua_system_path)
    if not os.path.isdir(lua_system_path):
        try:
            copytree(lua_src_path, lua_system_path)
        except OSError as e:
            lua_syspath_error(lua_system_path, 5)
    else:
        for fname in os.listdir(lua_src_path):
            try:
                copytree(os.path.join(lua_src_path, fname), lua_system_path)
            except OSError as e:
                if e.errno == errno.ENOTDIR:
                    try:
                        copy2(os.path.join(lua_src_path, fname), lua_system_path)
                    except:
                        lua_syspath_error(lua_system_path, 6)
                else:
                    lua_syspath_error(lua_system_path, 7)

################################################################################
# GENERIC BUILD SETTINGS
################################################################################

include_dirs = ['src/include'] + [x for x in os.getenv('CPATH', '').split(':') if len(x) > 0]
extra_compile_args = [
    '-std=gnu99', '-g', '-Wall', '-fPIC', '-O1',
    '-fno-common', '-fno-strict-aliasing', '-Wno-strict-prototypes',
    '-march=nocona',
    '-D_FILE_OFFSET_BITS=64', '-D_REENTRANT', '-D_GNU_SOURCE'
    ]
extra_objects = []
extra_link_args = []
library_dirs = []
libraries = [
  'ssl',
  'crypto',
  'pthread',
  'm',
  'z'
  ]

################################################################################
# PLATFORM SPECIFIC BUILD SETTINGS
################################################################################

if DARWIN:
    #---------------------------------------------------------------------------
    # Mac Specific Compiler and Linker Settings
    #---------------------------------------------------------------------------
    extra_compile_args = extra_compile_args + [
        '-D_DARWIN_UNLIMITED_SELECT',
        '-DMARCH_x86_64'
        ]
    if AEROSPIKE_C_HOME:
        PREFIX = AEROSPIKE_C_HOME + '/target/Darwin-x86_64'

elif LINUX:
    #---------------------------------------------------------------------------
    # Linux Specific Compiler and Linker Settings
    #---------------------------------------------------------------------------
    extra_compile_args = extra_compile_args + [
        '-rdynamic', '-finline-functions',
        '-DMARCH_x86_64'
        ]
    libraries = libraries + ['rt']
    if AEROSPIKE_C_HOME:
        PREFIX = AEROSPIKE_C_HOME + '/target/Linux-x86_64'
else:
    print("error: OS not supported:", PLATFORM, file=sys.stderr)
    sys.exit(8)

################################################################################
# RESOLVE C CLIENT DEPENDENCY AND LUA SYSTEM PATH
################################################################################

# Determine where the system lua files should be copied to
lua_system_path = ''
for arg in sys.argv:
    if arg[0:17] == '--lua-system-path':
        option, val = arg.split('=')
        lua_system_path = val.strip()
if not lua_system_path:
    lua_system_path = '/usr/local/aerospike/lua'

# If the C client is packaged elsewhere, assume the libraries are available
if os.environ.get('NO_RESOLVE_C_CLIENT_DEP', None):
    has_c_client = True
    libraries = libraries + ['aerospike']
    lua_src_path = os.environ.get('AEROSPIKE_LUA_PATH', lua_system_path)
else:
    has_c_client = False
    lua_src_path = "aerospike-client-c/lua"

data_files = [
    ('aerospike', []),
    ('aerospike/usr-lua', []),
    ('aerospike/lua', [
        lua_src_path + '/aerospike.lua',
        lua_src_path + '/as.lua',
        lua_src_path + '/stream_ops.lua'
        ]
    )
]

if not has_c_client:
    if ('build' in sys.argv or 'build_ext' in sys.argv or
        'install' in sys.argv):
        resolve_c_client(lua_src_path, lua_system_path)

################################################################################
# SETUP
################################################################################

# Get the long description from the relevant file
with open(os.path.join(CWD, 'README.rst')) as f:
    long_description = f.read()

# Get the version from the relevant file
with open(os.path.join(CWD, 'VERSION')) as f:
    version = f.read()

setup(
    cmdclass={
        'build': BuildCommand,
        'install': InstallCommand,
    },
    name = 'aerospike',
    version = version.strip(),
    description = 'Aerospike Client Library for Python',
    long_description = long_description,
    author = 'Aerospike, Inc.',
    author_email = 'info@aerospike.com',
    url = 'http://aerospike.com',
    license = 'Apache Software License',
    keywords = ['aerospike', 'nosql', 'database'],
    classifiers = [
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: POSIX :: Linux',
        'Operating System :: MacOS :: MacOS X',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Database'
    ],

    # Package Data Files
    zip_safe = False,
    include_package_data = True,
    package_data = {
        'aerospike': [
            lua_src_path + '/*.lua',
        ]
    },

    # Data files
    data_files = data_files,
    eager_resources = [
        lua_src_path + '/aerospike.lua',
        lua_src_path + '/as.lua',
        lua_src_path + '/stream_ops.lua',
    ],
    ext_modules = [
        Extension(
            # Extension Name
            'aerospike',

            # Source Files
            [
                'src/main/aerospike.c',
                'src/main/exception.c',
                'src/main/log.c',
                'src/main/client/type.c',
                'src/main/client/apply.c',
                'src/main/client/close.c',
                'src/main/client/connect.c',
                'src/main/client/exists.c',
                'src/main/client/exists_many.c',
                'src/main/client/get.c',
                'src/main/client/get_many.c',
                'src/main/client/select_many.c',
                'src/main/client/info_node.c',
                'src/main/client/info.c',
                'src/main/client/key.c',
                'src/main/client/put.c',
                'src/main/client/operate.c',
                'src/main/client/query.c',
                'src/main/client/remove.c',
                'src/main/client/scan.c',
                'src/main/client/select.c',
                'src/main/client/admin.c',
                'src/main/client/udf.c',
                'src/main/client/sec_index.c',
                'src/main/serializer.c',
                'src/main/client/remove_bin.c',
                'src/main/client/get_key_digest.c',
                'src/main/client/lstack.c',
                'src/main/client/lset.c',
                'src/main/client/lmap.c',
                'src/main/client/llist.c',
                'src/main/key/type.c',
                'src/main/key/apply.c',
                'src/main/key/exists.c',
                'src/main/key/get.c',
                'src/main/key/put.c',
                'src/main/key/remove.c',
                'src/main/query/type.c',
                'src/main/query/apply.c',
                'src/main/query/foreach.c',
                'src/main/query/results.c',
                'src/main/query/select.c',
                'src/main/query/where.c',
                'src/main/scan/type.c',
                'src/main/scan/foreach.c',
                'src/main/scan/results.c',
                'src/main/scan/select.c',
                'src/main/lstack/type.c',
                'src/main/lstack/lstack_operations.c',
                'src/main/lset/type.c',
                'src/main/lset/lset_operations.c',
                'src/main/llist/type.c',
                'src/main/llist/llist_operations.c',
                'src/main/lmap/type.c',
                'src/main/lmap/lmap_operations.c',
                'src/main/geospatial/type.c',
                'src/main/geospatial/wrap.c',
                'src/main/geospatial/unwrap.c',
                'src/main/geospatial/loads.c',
                'src/main/geospatial/dumps.c',
                'src/main/conversions.c',
                'src/main/policy.c',
                'src/main/predicates.c',
                'src/main/global_hosts/type.c',
                'src/main/nullobject/type.c',
            ],

            # Compile
            include_dirs = include_dirs,
            extra_compile_args = extra_compile_args,

            # Link
            library_dirs = library_dirs,
            libraries = libraries,
            extra_objects = extra_objects,
            extra_link_args = extra_link_args,
        )
    ]
)




# -*- coding: utf-8 -*-

import pytest
import sys
import time
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)


class TestLList(TestBaseClass):

    pytestmark = pytest.mark.skipif(
        TestBaseClass.has_ldt_support() == False,
        reason="LDTs are not enabled for namespace 'test'")

    llist_integer = None
    llist_string = None
    client = None
    key1 = None
    key2 = None

    def setup_class(self):

        print "setup class invoked..."
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            self.client = aerospike.client(config).connect()
        else:
            self.client = aerospike.client(config).connect(user, password)

        TestLList.key1 = ('test', 'demo', 'integer_llist_ky')
        TestLList.llist_integer = TestLList.client.llist(TestLList.key1,
                                                         'integer_bin')
        TestLList.key2 = ('test', 'demo', 'string_llist_ky')
        TestLList.llist_string = TestLList.client.llist(TestLList.key2,
                                                        'string_bin')
        TestLList.key3 = ('test', 'demo', 'float_llist_ky')
        TestLList.llist_float = TestLList.client.llist(TestLList.key3,
                                                       'float_bin')


    def teardown_class(self):
        print "teardown class invoked..."
        try:
            TestLList.llist_integer.destroy()
            TestLList.llist_string.destroy()
            TestLList.list_float.destroy()
        except:
            pass
        self.client.close()

    #Add() - Add an object to the llist.
    #Get() - Get an object from the llist.
    #Size() - Get the current item count of the llist.
    def test_llist_add_get_size_positive(self):
        """
            Invoke add() an object to LList.
        """

        assert 0 == TestLList.llist_integer.add(11)
        assert [11] == TestLList.llist_integer.get(11)

        assert 0 == TestLList.llist_string.add("abc")
        assert ['abc'] == TestLList.llist_string.get('abc')

        assert 1 == TestLList.llist_integer.size()

    #Add() - Add() unsupported type data to llist.
    def test_llist_add_float_positive(self):
        """
            Invoke add() float type data.
        """
        rec = {"pi": 3.14}

        try:
            TestLList.llist_float.add(rec)

        except LDTKeyFunctionNotFound as exception:
            assert exception.code == 1433
            assert exception.msg == "LDT-Key Field Not Found"

    #Add() - Add() without any mandatory parameters.
    def test_llist_no_parameter_negative(self):
        """
            Invoke add() without any mandatory parameters.
        """

        with pytest.raises(TypeError) as typeError:
            TestLList.llist_integer.add()

    #Add_many() - Add a list of objects to the set.
    def test_llist_add_many_positive(self):
        """
            Invoke add_many() to add a list of objects to the set.
        """

        policy = {'timeout': 7000}
        assert 0 == TestLList.llist_integer.add_many([122, 56, 871], policy)

        assert [122] == TestLList.llist_integer.get(122)
        assert [56] == TestLList.llist_integer.get(56)
        assert [871] == TestLList.llist_integer.get(871)

    #Get() - Get without any mandatory parameters.
    def test_llist_get_element_negative(self):
        """
            Invoke get() without any mandatory parameters.
        """

        with pytest.raises(TypeError) as typeError:
            TestLList.llist_integer.get()

    #Remove() and Get()- Remove an object from the set and get non-existent element.
    def test_llist_remove_positive(self):
        """
            Invoke remove() to remove element.
        """

        assert 0 == TestLList.llist_string.add('remove')
        assert 0 == TestLList.llist_string.remove('remove')

        try:
            TestLList.llist_string.get('remove')

        except UDFError as exception:
            assert exception.code == 100L
        except LargeItemNotFound as exception:
            assert exception.code == 125L

    #Remove() - Remove non-existent object from the llist.
    def test_llist_remove_element_negative(self):
        """
            Invoke remove() to remove non-existent element.
        """

        try:
            TestLList.llist_string.remove('kk')

        except UDFError as exception:
            assert exception.code == 100L
        except LargeItemNotFound as exception:
            assert exception.code == 125L

    #Destroy() - Delete the entire LList(LDT Remove).
    def test_llist_destroy_positive(self):
        """
            Invoke destroy() to delete entire LDT.
        """
        key = ('test', 'demo', 'remove')
        llist = self.client.llist(key, 'llist_add')
        try:
            llist.add(876)
        except:
            pass
        assert 0 == llist.destroy()

    def test_llist_ldt_initialize_negative(self):
        """
            Initialize ldt with wrong key.
        """
        key = ('test', 'demo', 12.3)
        try:
            llist = self.client.llist(key, 'ldt_stk')
        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Parameters are incorrect"

    def test_llist_find_first_positive_without_policy(self):
        """
            Invoke find_first() to access elements
        """
        elements_list = TestLList.llist_integer.find_first(2)

        assert elements_list == [11, 56]

    def test_llist_find_first_positive(self):
        """
            Invoke find_first() to access elements
        """
        elements_list = TestLList.llist_integer.find_first(2, {'timeout': 1000})
        assert elements_list == [11, 56]

    def test_llist_find_first_count_large_positive(self):
        """
            Invoke find_first() to access elements with a larger count
        """
        elements_list = TestLList.llist_integer.find_first(10, {'timeout': 1000})
        assert elements_list == [11, 56, 122, 871]

    def test_llist_find_first_count_negative(self):
        """
            Invoke find_first() to access elements with a negative count
        """
        elements_list = TestLList.llist_integer.find_first(-8, {'timeout': 1000})
        assert elements_list == [11, 56, 122, 871]

    def test_llist_find_last_positive_without_policy(self):
        """
            Invoke find_last() to access elements
        """
        elements_list = TestLList.llist_integer.find_last(2)
        assert elements_list == [871, 122]

    def test_llist_find_last_positive(self):
        """
            Invoke find_last() to access elements
        """
        elements_list = TestLList.llist_integer.find_last(2, {'timeout': 1000})
        assert elements_list == [871, 122]

    def test_llist_find_last_count_large(self):
        """
            Invoke find_last() to access elements
        """
        elements_list = TestLList.llist_integer.find_last(15, {'timeout': 1000})
        assert elements_list == [871, 122, 56, 11]

    def test_llist_find_last_count_negative(self):
        """
            Invoke find_last() to access elements
        """
        elements_list = TestLList.llist_integer.find_last(-2, {'timeout': 1000})
        assert elements_list == [871, 122, 56, 11]

    def test_llist_find_last_no_params(self):
        """
            Invoke find_last() to access elements
        """
        with pytest.raises(TypeError) as typeError:
            TestLList.llist_integer.find_last()
        assert "Required argument 'count' (pos 1) not found" in typeError.value

    def test_llist_find_last_no_parameters_negative(self):
        """
            Invoke find_last() to access elements
        """
        with pytest.raises(TypeError) as typeError:
            TestLList.llist_integer.find_last()
        assert "Required argument 'count' (pos 1) not found" in typeError.value

    def test_llist_find_from_positive_without_policy(self):
        """
            Invoke find_from() to access elements from a given key
        """
        elements_list = TestLList.llist_integer.find_from(56, 2)
        assert elements_list == [56, 122]

    def test_llist_find_from_positive(self):
        """
            Invoke find_from() to access elements from a given key
        """
        elements_list = TestLList.llist_integer.find_from(56, 2, {'timeout': 1000})
        assert elements_list == [56, 122]

    def test_llist_find_from_positive_non_existent_key(self):
        """
            Invoke find_from() to access elements from a non-existent key
        """
        elements_list = TestLList.llist_integer.find_from(21, 2, {'timeout': 1000})
        assert elements_list == [56, 122]

    def test_llist_range_limit_positive_without_policy(self):
        """
            Invoke range_limit() to access elements
        """
        elements_list = TestLList.llist_integer.range_limit(56, 871, 2, None, None)
        assert elements_list == [56, 122, 871]

    def test_llist_range_limit_positive(self):
        """
            Invoke range_limit() to access elements
        """
        elements_list = TestLList.llist_integer.range_limit(56, 871, 2, None, None, {'timeout': 1000})
        assert elements_list == [56, 122, 871]

    def test_llist_range_limit_negative_keys(self):
        """
            Invoke range_limit() to access elements with negative keys
        """
        elements_list = TestLList.llist_integer.range_limit(-56, -871, 2, None, None, {'timeout': 1000})
        assert elements_list == []

    def test_llist_range_limit_larger_count_positive(self):
        """
            Invoke range_limit() to access elements with larger count than list
            size
        """
        elements_list = TestLList.llist_integer.range_limit(56, 871, 8, None, None, {'timeout': 1000})
        assert elements_list == [56, 122, 871]

    def test_llist_range_limit_count_negative(self):
        """
            Invoke range_limit() to access elements
        """
        elements_list = TestLList.llist_integer.range_limit(56, 871, -2, None, None, {'timeout': 1000})
        assert elements_list == [56, 122, 871]

    def test_llist_set_page_size_without_policy(self):
        #Invoke set_page_size() to set page size of ldt bin.
        assert 0 == TestLList.llist_integer.set_page_size(8192)

    def test_llist_set_page_size(self):
        #Invoke set_page_size() to set page size of ldt bin.
        assert 0 == TestLList.llist_integer.set_page_size(8192, {'timeout': 0})

    def test_llist_set_page_size_string_negative(self):
        with pytest.raises(TypeError) as typeError:
            TestLList.llist_integer.set_page_size("8192", {'timeout': 0})
        assert "an integer is required" in typeError.value

    """ Causes db to shutdown
    def test_llist_find_from_positive_negative_count(self):
        #Invoke find_from() to access elements with a negative count
        elements_list = TestLList.llist_integer.find_from(56, -2, {'timeout': 1000})

        assert elements_list == [56, 122]
        """



# -*- coding: utf-8 -*-

import pytest
import sys
import time
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestDropRole(TestBaseClass):

    pytestmark = pytest.mark.skipif(
        TestBaseClass().get_hosts()[1] == None,
        reason="No user specified, may be not secured cluster.")

    def setup_method(self, method):

        """
        Setup method
        """
        hostlist, user, password = TestBaseClass().get_hosts()
        config = {
                "hosts": hostlist
                }
        self.client = aerospike.client(config).connect( user, password )

        self.delete_users = []

    def teardown_method(self, method):

        """
        Teardown method
        """

        policy = {}

        for user in self.delete_users:
            self.client.admin_drop_user( user, policy )

        self.client.close()

    def test_drop_role_without_any_parameters(self):

        with pytest.raises(TypeError) as typeError:
            self.client.admin_drop_role()

        assert "Required argument 'role' (pos 1) not found" in typeError.value

    def test_drop_role_positive_with_policy(self):
        """
            Drop role positive with policy
        """
        try:
            self.client.admin_query_role("usr-sys-admin-test")
            # role exists, clear it out.
            self.client.admin_drop_role("usr-sys-admin-test")
        except InvalidRole:
            pass # we are good, no such role exists

        self.client.admin_create_role("usr-sys-admin-test", [{"code":
            aerospike.PRIV_READ, "ns": "test", "set":"demo"}], {'timeout': 1000})
        time.sleep(1)
        roles = self.client.admin_query_role("usr-sys-admin-test")
        assert roles == [{"code": 10, "ns": "test", "set": "demo"}]

        try:
            self.client.admin_query_user("testcreaterole")
            # user exists, clear it out.
            self.client.admin_drop_user("testcreaterole")
        except AdminError:
            pass # we are good, no such user exists

        status = self.client.admin_create_user("testcreaterole", "createrole",
            ["usr-sys-admin-test"])

        assert status == 0
        time.sleep(1)
        users = self.client.admin_query_user("testcreaterole")

        assert users == ["usr-sys-admin-test"]

        status = self.client.admin_drop_role("usr-sys-admin-test", {'timeout': 1000})

        assert status == 0

        users = self.client.admin_query_user("testcreaterole")

        assert users == []

        self.client.admin_drop_user("testcreaterole")

    def test_drop_role_positive(self):
        """
            Drop role positive
        """
        try:
            self.client.admin_query_role("usr-sys-admin-test")
            # role exists, clear it out.
            self.client.admin_drop_role("usr-sys-admin-test")
        except InvalidRole:
            pass # we are good, no such role exists

        status = self.client.admin_create_role("usr-sys-admin-test", [{"code":
            aerospike.PRIV_USER_ADMIN}, {"code": aerospike.PRIV_SYS_ADMIN}])
        time.sleep(1)
        roles = self.client.admin_query_role("usr-sys-admin-test")
        assert roles == [{"code": 0, "ns": "", "set": ""}, {"code": 1, "ns": "", "set": ""}]

        try:
            self.client.admin_query_user("testcreaterole")
            # user exists, clear it out.
            self.client.admin_drop_user("testcreaterole")
        except AdminError:
            pass # we are good, no such user exists

        status = self.client.admin_create_user("testcreaterole", "createrole",
["usr-sys-admin-test"])

        assert status == 0
        time.sleep(1)
        users = self.client.admin_query_user("testcreaterole")

        assert users == ["usr-sys-admin-test"]

        status = self.client.admin_drop_role("usr-sys-admin-test")

        assert status == 0

        users = self.client.admin_query_user("testcreaterole")

        assert users == []

        self.client.admin_drop_user("testcreaterole")

    def test_drop_non_existent_role(self):
        """
            Drop non-existent role
        """
        try:
            self.client.admin_drop_role("usr-sys-admin-test")

        except InvalidRole as exception:
            assert exception.code == 70
            assert exception.msg == "AEROSPIKE_INVALID_ROLE"

    def test_drop_role_rolename_None(self):
        """
            Drop role with role name None
        """
        try:
            self.client.admin_drop_role(None)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Role name should be a string"

    def test_drop_role_with_incorrect_policy(self):
        """
            Drop role with incorrect policy
        """
        status = self.client.admin_create_role("usr-sys-admin-test", [{"code":
            aerospike.PRIV_USER_ADMIN}])

        assert status == 0

        try:
            self.client.admin_drop_role("usr-sys-admin-test", {"timeout": 0.2})

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'timeout is invalid'

        self.client.admin_drop_role("usr-sys-admin-test")



# -*- coding: utf-8 -*-
import pytest
import time
import sys
import cPickle as pickle
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestGetNodes(object):
    def setup_class(cls):
        """
        Setup class.
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            TestGetNodes.client = aerospike.client(config).connect()
        else:
            TestGetNodes.client = aerospike.client(config).connect(user,
                                                                   password)

    def teardown_class(cls):
        """
        Teardown class.
        """
        TestGetNodes.client.close()

    def test_get_nodes_positive(self):

        response = TestGetNodes.client.get_nodes()
        assert response != None

    def test_get_nodes_with_parameter(self):

        response = TestGetNodes.client.get_nodes("parameter")
        assert response != None

    def test_get_nodes_positive_without_connection(self):
        config = {'hosts': [('127.0.0.1', 3000)]}
        client1 = aerospike.client(config)

        try:
            response = client1.get_nodes()

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'



# -*- coding: utf-8 -*-

import pytest
import sys
import time
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestUdfPut(TestBaseClass):
    def setup_class(cls):
        """
        Setup class
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}

        if user == None and password == None:
            TestUdfPut.client = aerospike.client(config).connect()
        else:
            TestUdfPut.client = aerospike.client(config).connect(user, password)

    def teardown_class(cls):
        TestUdfPut.client.close()

    def setup_method(self, method):
        """
        Setup method
        """

    def teardown_method(self, method):
        """
        Teardown method
        """
        time.sleep(1)
        udf_list = TestUdfPut.client.udf_list({'timeout': 100})
        for udf in udf_list:
            if udf['name'] == 'example.lua':
                TestUdfPut.client.udf_remove("example.lua")

    def test_udf_put_without_parameters(self):

        with pytest.raises(TypeError) as typeError:
            TestUdfPut.client.udf_put()

        assert "Required argument 'filename' (pos 1) not found" in typeError.value

    def test_udf_put_with_proper_parameters(self):

        policy = {}
        filename = "example.lua"
        udf_type = 0

        status = TestUdfPut.client.udf_put(filename, udf_type, policy)

        assert status == 0
        udf_list = TestUdfPut.client.udf_list({})

        present = False
        for udf in udf_list:
            if 'example.lua' == udf['name']:
                present = True

        assert True if present else False

    def test_udf_put_with_invalid_timeout_policy_value(self):

        policy = {'timeout': 0.1}
        filename = "example.lua"
        udf_type = 0

        try:
            status = TestUdfPut.client.udf_put( filename, udf_type, policy )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "timeout is invalid"

    def test_udf_put_with_proper_timeout_policy_value(self):

        policy = {'timeout': 1000}
        filename = "example.lua"
        udf_type = 0

        status = TestUdfPut.client.udf_put(filename, udf_type, policy)

        assert status == 0

        udf_list = TestUdfPut.client.udf_list({})

        present = False
        for udf in udf_list:
            if 'example.lua' == udf['name']:
                present = True

    def test_udf_put_with_non_existent_filename(self):

        policy = {}
        filename = "somefile"
        udf_type = 0

        try:
            TestUdfPut.client.udf_put( filename, udf_type, policy )
        except LuaFileNotFound as e:
            assert e.code == 1302

    def test_udf_put_with_non_lua_udf_type_and_lua_script_file(self):

        policy = {'timeout': 0}
        filename = "example.lua"
        udf_type = 1

        try:
            status = TestUdfPut.client.udf_put( filename, udf_type, policy )

        except ClientError as exception:
            assert exception.code == -1
            assert exception.msg == "Invalid UDF language"

    def test_udf_put_with_all_none_parameters(self):

        with pytest.raises(TypeError) as exception:
            status = TestUdfPut.client.udf_put(None, None, None)

        assert "an integer is required" in exception.value

    def test_udf_put_with_filename_unicode(self):

        policy = {}
        filename = u"example.lua"
        udf_type = 0

        status = TestUdfPut.client.udf_put(filename, udf_type, policy)

        assert status == 0
        time.sleep(2)
        udf_list = TestUdfPut.client.udf_list({})

        present = False
        for udf in udf_list:
            if 'example.lua' == udf['name']:
                present = True

        assert True if present else False

    def test_udf_put_with_proper_parameters_without_connection(self):

        policy = {}
        filename = "example.lua"
        udf_type = 0

        config = {'hosts': [('127.0.0.1', 3000)]}

        client1 = aerospike.client(config)

        try:
            status = client1.udf_put( filename, udf_type, policy )

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'



# -*- coding: utf-8 -*-

import pytest
import sys
import time
import json
import marshal
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class SomeClass(object):

    pass

def serialize_function_old_server(val):
    return marshal.dumps(val)

def serialize_function(val):
    return json.dumps(marshal.dumps(val))

def client_serialize_function(val):
    return marshal.dumps(val)

def deserialize_function(val):
    return marshal.loads(json.loads(val))

def deserialize_function_old_server(val):
    return marshal.loads(val)

def client_deserialize_function(val):
    return marshal.loads(val)


class TestUserSerializer(object):
    def setup_class(cls):
        """
            Setup class
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            TestUserSerializer.client = aerospike.client(config).connect()
        else:
            TestUserSerializer.client = aerospike.client(config).connect(user, password)
        TestUserSerializer.skip_old_server = True
        versioninfo = TestUserSerializer.client.info('version')
        for keys in versioninfo:
            for value in versioninfo[keys]:
                if value != None:
                    versionlist = value[value.find("build") + 6:value.find("\n")].split(".")
                    if int(versionlist[0]) >= 3 and int(versionlist[1]) >= 6:
                        TestUserSerializer.skip_old_server = False

    def teardown_class(cls):
        TestUserSerializer.client.close()

    def setup_method(self, method):
        """
            Setup method
        """

        self.delete_keys = []

    def teardown_method(self, method):
        """
            Teardown method
        """
        for key in self.delete_keys:
            TestUserSerializer.client.remove(key)


    def test_put_with_float_data_user_serializer(self):

        #    Invoke put() for float data record with user serializer.

        if TestUserSerializer.skip_old_server == False:
            response = aerospike.set_serializer(serialize_function)
            response = aerospike.set_deserializer(deserialize_function)
        else:
            response = aerospike.set_serializer(serialize_function_old_server)
            response = aerospike.set_deserializer(deserialize_function_old_server)
        key = ('test', 'demo', 1)

        rec = {"pi": 3.14}

        res = TestUserSerializer.client.put(key, rec, {}, {},
                                        aerospike.SERIALIZER_USER)

        assert res == 0

        _, _, bins = TestUserSerializer.client.get(key)

        assert bins == {'pi': 3.14}

        self.delete_keys.append(key)


    def test_put_with_bool_data_user_serializer(self):
        """
            Invoke put() for bool data record with user serializer.
        """

        response = aerospike.set_serializer(serialize_function)
        response = aerospike.set_deserializer(deserialize_function)
        key = ('test', 'demo', 1)

        rec = {'status': True}

        res = TestUserSerializer.client.put(key, rec, {}, {},
                                        aerospike.SERIALIZER_USER)

        assert res == 0

        _, _, bins = TestUserSerializer.client.get(key)

        assert bins == {'status': True}

        self.delete_keys.append(key)


    """
    def test_put_with_object_data_user_serializer(self):

            #Invoke put() for object data record with user serializer.

        key = ( 'test', 'demo', 1 )

        obj1 = SomeClass()
        rec = {
            'object': obj1
        }

        res = TestUserSerializer.client.put( key, rec , {}, {}, aerospike.SERIALIZER_USER)

        assert res == 0

        _, _, bins = TestUserSerializer.client.get( key )

        #assert bins == { 'object': True }
        assert bins == { 'object': obj1 }

        self.delete_keys.append( key )

    def test_put_with_object_data_python_serializer(self):

            Invoke put() for object data record with python serializer.

        key = ( 'test', 'demo', 1 )

        obj1 = SomeClass()
        rec = {
                "object" : obj1
        }

        res = TestUserSerializer.client.put( key, rec , {}, {}, aerospike.SERIALIZER_PYTHON)

        assert res == 0
        _, _, bins = TestUserSerializer.client.get( key )

        assert bins == { 'object': True }

        self.delete_keys.append( key )
    """

    def test_put_with_float_data_user_serializer_none(self):
        """
            Invoke put() for float data record with user serializer.
        """

        try:
            response = aerospike.set_serializer(None)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Parameter must be a callable"

    def test_put_with_float_data_user_deserializer_none(self):
        """
            Invoke put() for float data record with user deserializer None.
        """

        if TestUserSerializer.skip_old_server == False:
            response = aerospike.set_serializer(serialize_function)
        else:
            response = aerospike.set_serializer(serialize_function_old_server)

        key = ('test', 'demo', 1)

        rec = {"pi": 3.14}

        res = TestUserSerializer.client.put(key, rec, {}, {},
                                        aerospike.SERIALIZER_USER)

        assert res == 0

        self.delete_keys.append(key)

        try:
            response = aerospike.set_deserializer(None)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Parameter must be a callable"

    def test_put_with_mixed_data_user_serializer(self):
        """
            Invoke put() for mixed data record with user serializer.
        """

        key = ('test', 'demo', 1)

        if TestUserSerializer.skip_old_server == False:
            response = aerospike.set_serializer(serialize_function)
            response = aerospike.set_deserializer(deserialize_function)
        else:
            response = aerospike.set_serializer(serialize_function_old_server)
            response = aerospike.set_deserializer(deserialize_function_old_server)
        rec = {
            'map': {"key": "asd';q;'1';",
                    "pi": 3.14},
            'normal': 1234,
            'special': '!@#@#$QSDAsd;as',
            'list': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8")],
            'bytes': bytearray("asd;as[d'as;d", "utf-8"),
            'nestedlist': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8"),
                           [1, bytearray("asd;as[d'as;d", "utf-8")]],
            'nestedmap': {
                "key": "asd';q;'1';",
                "pi": 3.14,
                "nest": {"pi1": 3.12,
                         "t": 1}
            },
        }
        res = TestUserSerializer.client.put(key, rec, {}, {},
                                        aerospike.SERIALIZER_USER)

        assert res == 0

        _, _, bins = TestUserSerializer.client.get(key)

        assert bins == {
            'map': {"key": "asd';q;'1';",
                    "pi": 3.14},
            'normal': 1234,
            'special': '!@#@#$QSDAsd;as',
            'list': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8")],
            'bytes': bytearray("asd;as[d'as;d", "utf-8"),
            'nestedlist': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8"),
                           [1, bytearray("asd;as[d'as;d", "utf-8")]],
            'nestedmap':
            {"key": "asd';q;'1';",
             "pi": 3.14,
             "nest": {"pi1": 3.12,
                      "t": 1}},
        }

        self.delete_keys.append(key)

    def test_put_with_mixeddata_client_serializer_deserializer_with_spec_in_put(self):

        #    Invoke put() for mixed data with class and instance serialziers
        #    with a specification in put. Client one is called

        hostlist, user, password = TestBaseClass.get_hosts()
        method_config = {'hosts': hostlist,
                'serialization': (client_serialize_function,
                    client_deserialize_function)}
        if user == None and password == None:
            client = aerospike.client(method_config).connect()
        else:
            client = aerospike.client(method_config).connect(user, password)
        response = aerospike.set_serializer(serialize_function)
        response = aerospike.set_deserializer(deserialize_function)
        key = ('test', 'demo', 1)

        rec = {
            'map': {"key": "asd';q;'1';",
                    "pi": 3},
            'normal': 1234,
            'special': '!@#@#$QSDAsd;as',
            'list': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8")],
            'bytes': bytearray("asd;as[d'as;d", "utf-8"),
            'nestedlist': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8"),
                           [1, bytearray("asd;as[d'as;d", "utf-8")]],
            'nestedmap': {
                "key": "asd';q;'1';",
                "pi": 314,
                "nest": {"pi1": 312,
                         "t": 1}
            },
        }

        res = client.put(key, rec, {}, {}, aerospike.SERIALIZER_USER)

        assert res == 0

        _, _, bins = client.get(key)


        assert bins == {
            'map': {"key": "asd';q;'1';",
                    "pi": 3},
            'normal': 1234,
            'special': '!@#@#$QSDAsd;as',
            'list': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8")],
            'bytes': bytearray("asd;as[d'as;d", "utf-8"),
            'nestedlist': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8"),
                           [1, bytearray("asd;as[d'as;d", "utf-8")]],
            'nestedmap': {
                "key": "asd';q;'1';",
                "pi": 314,
                "nest": {"pi1": 312,
                         "t": 1}
            },
        }
        client.close()

        self.delete_keys.append(key)

    def test_put_with_mixeddata_client_serializer_deserializer_no_put_spec(self):

        #    Invoke put() for mixed data with class and instance serialziers
        #    with no specification in put
        hostlist, user, password = TestBaseClass.get_hosts()
        method_config = {'hosts': hostlist,
                'serialization': (client_serialize_function,
                    client_deserialize_function)}
        if user == None and password == None:
            client = aerospike.client(method_config).connect()
        else:
            client = aerospike.client(method_config).connect(user, password)

        response = aerospike.set_serializer(serialize_function)
        response = aerospike.set_deserializer(deserialize_function)
        key = ('test', 'demo', 1)

        rec = {
            'map': {"key": "asd';q;'1';",
                    "pi": 3},
            'normal': 1234,
            'special': '!@#@#$QSDAsd;as',
            'list': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8")],
            'bytes': bytearray("asd;as[d'as;d", "utf-8"),
            'nestedlist': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8"),
                           [1, bytearray("asd;as[d'as;d", "utf-8")]],
            'nestedmap': {
                "key": "asd';q;'1';",
                "pi": 3.14,
                "nest": {"pi1": 312,
                         "t": 1}
            },
        }

        res = client.put(key, rec, {}, {})

        assert res == 0

        _, _, bins = client.get(key)

        assert bins == {
            'map': {"key": "asd';q;'1';",
                    "pi": 3},
            'normal': 1234,
            'special': '!@#@#$QSDAsd;as',
            'list': ["nanslkdl", 1, "asd;as[d'as;d"],
            'bytes': "asd;as[d'as;d",
            'nestedlist': ["nanslkdl", 1, "asd;as[d'as;d",
                           [1, "asd;as[d'as;d"]],
            'nestedmap': {
                "key": "asd';q;'1';",
                "pi": 3.14,
                "nest": {"pi1": 312,
                         "t": 1}
            },
        }
        client.close()
        self.delete_keys.append(key)



# -*- coding: utf-8 -*-

import pytest
import sys
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestScan(TestBaseClass):

    def setup_method(self, method):
        """
        Setup method.
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            self.client = aerospike.client(config).connect()
        else:
            self.client = aerospike.client(config).connect(user, password)

        for i in xrange(20):
            key = ('test', u'demo', i)
            rec = {'name': 'name%s' % (str(i)), 'age': i}
            self.client.put(key, rec)

        key = ('test', u'demo', 122)
        list = [{"op": aerospike.OPERATOR_APPEND,
                "bin": bytearray("asd;adk\0kj", "utf-8"),
                "val": u"john"}];
        self.client.operate(key, list)

    def teardown_method(self, method):
        """
        Teardown method
        """

        for i in xrange(20):
            key = ('test', u'demo', i)
            self.client.remove(key)

        key = ('test', 'demo', 122)
        self.client.remove(key)
        self.client.close()

    def test_scan_without_any_parameter(self):

        scan_obj = None
        with pytest.raises(TypeError) as typeError:
            scan_obj = self.client.scan()
            scan_obj.foreach()

        assert "Required argument 'callback' (pos 1) not found" in typeError.value

    def test_scan_with_non_existent_ns_and_set(self):

        ns = 'namespace'
        st = 'set'

        records = []
        scan_obj = None
        scan_obj = self.client.scan(ns, st)

        def callback((key, meta, bins)):
            records.append(bins)

        try:
            scan_obj.foreach(callback)

        except NamespaceNotFound as exception:
            assert exception.code == 20L
        except ServerError as exception:
            assert exception.code == 1L

    def test_scan_with_none_ns_and_set(self):

        ns = None
        st = None

        try:
            scan_obj = self.client.scan( ns, st )

        except ParamError as exception:
            assert exception.code == -2L
            assert exception.msg == 'Parameters are incorrect'

    def test_scan_with_existent_ns_and_set(self):

        ns = 'test'
        st = 'demo'

        records = []
        scan_obj = None

        def callback((key, meta, bins)):
            records.append(bins)

        scan_obj = self.client.scan(ns, st)

        scan_obj.foreach(callback)

        assert len(records) != 0

    def test_scan_with_existent_ns_and_none_set(self):

        ns = 'test'
        st = 'demo'

        records = []
        scan_obj = None

        def callback((key, meta, bins)):
            records.append(bins)

        scan_obj = self.client.scan(ns, None)

        scan_obj.foreach(callback)

        assert len(records) != 0

    def test_scan_with_timeout_policy(self):

        ns = 'test'
        st = 'demo'

        records = []

        scan_obj = None

        def callback((key, meta, bins)):
            records.append(bins)

        scan_obj = self.client.scan(ns, st)

        scan_obj.foreach(callback, {'timeout': 2000})

        assert len(records) != 0

    def test_scan_with_callback_contains_error(self):
        pytest.xfail("segfault!")
        ns = 'test'
        st = 'demo'

        records = []
        scan_obj = None
        val = 1
        def callback( (key, meta, bins) ):
            val += 1
            records.append(bins)

        scan_obj = self.client.scan(ns, st)

        try:
            scan_obj.foreach(callback, { 'timeout' : 1000 })

        except ClientError as exception:
            assert exception.code == -1L
            assert exception.msg == "Callback function contains an error"

    def test_scan_with_callback_returning_false(self):
        """
            Invoke scan() with callback function returns false
        """
        ns = 'test'
        st = 'demo'

        records = []

        scan_obj = None

        def callback((key, meta, bins)):
            if len(records) == 10:
                return False
            records.append(bins)

        scan_obj = self.client.scan(ns, st)

        scan_obj.foreach(callback, {'timeout': 1000})
        assert len(records) == 10

    def test_scan_with_unicode_set(self):

        ns = 'test'

        st = u'demo'

        records = []

        scan_obj = None

        def callback((key, meta, bins)):
            records.append(bins)

        scan_obj = self.client.scan(ns, st)

        scan_obj.foreach(callback)

        assert len(records) != 0

    def test_scan_with_select_clause(self):

        ns = 'test'
        st = 'demo'

        records = []

        scan_obj = None

        def callback((key, meta, bins)):
            records.append(bins)

        scan_obj = self.client.scan(ns, st)

        scan_obj.select('name')

        scan_obj.foreach(callback)

        assert len(records) != 0

    def test_scan_with_results_method(self):

        ns = 'test'
        st = 'demo'

        records = []

        scan_obj = None
        scan_obj = self.client.scan(ns, st)

        scan_obj.select(u'name', u'age')

        records = scan_obj.results()

        assert len(records) != 0

    def test_scan_with_select_bin_integer(self):
        """
            Invoke scan() with select bin is of type integer.
        """
        scan_obj = None
        scan_obj = self.client.scan('test', 'demo')

        try:
            scan_obj.select(22, 'test_age')

        except ParamError as exception:
            assert exception.code == -2L
            assert exception.msg == 'Bin name should be of type string'

    def test_scan_with_options_positive(self):
        """
            Invoke scan() with options positive
        """
        ns = 'test'
        st = 'demo'

        records = []

        scan_obj = None
        options = {
            "percent": 100,
            "concurrent": True,
            "priority": aerospike.SCAN_PRIORITY_HIGH
        }

        def callback((key, meta, bins)):
            records.append(bins)

        scan_obj = self.client.scan(ns, st)

        scan_obj.foreach(callback, {}, options)

        assert len(records) != 0

    def test_scan_with_options_percent_negative(self):
        """
            Invoke scan() with options negative
        """
        ns = 'test'
        st = 'demo'

        records = []

        scan_obj = None
        options = {
            "percent": 80,
            "concurrent": True,
            "priority": aerospike.SCAN_PRIORITY_HIGH
        }

        def callback((key, meta, bins)):
            records.append(bins)

        scan_obj = self.client.scan(ns, st)

        scan_obj.foreach(callback, {}, options)

        assert records == []

    def test_scan_with_options_nobins(self):
        """
            Invoke scan() with nobins
        """
        ns = 'test'
        st = 'demo'

        records = []

        scan_obj = None
        options = {"priority": aerospike.SCAN_PRIORITY_HIGH, "nobins": True}

        def callback((key, meta, bins)):
            records.append(bins)

        scan_obj = self.client.scan(ns, st)

        scan_obj.foreach(callback, {}, options)

        assert len(records) != 0

    def test_scan_with_options_nobins_false(self):
        """
            Invoke scan() with nobins
        """
        ns = 'test'
        st = 'demo'

        records = []

        scan_obj = None
        options = {"priority": aerospike.SCAN_PRIORITY_HIGH, "nobins": "true"}

        def callback((key, meta, bins)):
            records.append(bins)

        scan_obj = self.client.scan(ns, st)

        try:
            scan_obj.foreach(callback, { 'timeout' : 1000 }, options)

        except ParamError as exception:
            assert exception.code == -2L
            assert exception.msg == 'Invalid value(type) for nobins'

    def test_scan_with_multiple_foreach_on_same_scan_object(self):
        """
            Invoke multiple foreach on same scan object.
        """
        ns = 'test'
        st = 'demo'

        records = []

        scan_obj = None

        def callback((key, meta, bins)):
            records.append(bins)

        scan_obj = self.client.scan(ns, st)

        scan_obj.foreach(callback)

        assert len(records) != 0

        records = []
        scan_obj.foreach(callback)

        assert len(records) != 0

    def test_scan_with_multiple_results_call_on_same_scan_object(self):

        ns = 'test'
        st = 'demo'

        scan_obj = None
        scan_obj = self.client.scan(ns, st)

        scan_obj.select(u'name', u'age')

        records = []
        records = scan_obj.results()
        assert len(records) != 0

        records = []
        records = scan_obj.results()
        assert len(records) != 0

    def test_scan_with_select_binnames_bytearray(self):

        ns = 'test'
        st = 'demo'

        records = []

        def callback((key, meta, bins)):
            records.append(bins)

        scan_obj = self.client.scan(ns, st)

        scan_obj.select(bytearray("asd;adk\0kj", "utf-8"))

        scan_obj.foreach(callback)

        assert len(records) == 1
